@@ -3,7 +3,7 @@ use std::hint::unreachable_unchecked;
 
 use crate::bitmap::utils::{BitmapIter, ZipValidity};
 use crate::bitmap::Bitmap;
-use crate::datatypes::{DataType, IntegerType};
+use crate::datatypes::{ArrowDataType, IntegerType};
 use crate::scalar::{new_scalar, Scalar};
 use crate::trusted_len::TrustedLen;
 use crate::types::NativeType;
@@ -37,6 +37,7 @@ pub unsafe trait DictionaryKey: NativeType + TryInto<usize> + TryFrom<usize> + H
     const KEY_TYPE: IntegerType;
 
     /// Represents this key as a `usize`.
+    ///
     /// # Safety
     /// The caller _must_ have checked that the value can be casted to `usize`.
     #[inline]
@@ -104,17 +105,17 @@ unsafe impl DictionaryKey for u64 {
 /// use `unchecked` calls to retrieve the values
 #[derive(Clone)]
 pub struct DictionaryArray<K: DictionaryKey> {
-    data_type: DataType,
+    data_type: ArrowDataType,
     keys: PrimitiveArray<K>,
     values: Box<dyn Array>,
 }
 
 fn check_data_type(
     key_type: IntegerType,
-    data_type: &DataType,
-    values_data_type: &DataType,
+    data_type: &ArrowDataType,
+    values_data_type: &ArrowDataType,
 ) -> PolarsResult<()> {
-    if let DataType::Dictionary(key, value, _) = data_type.to_logical_type() {
+    if let ArrowDataType::Dictionary(key, value, _) = data_type.to_logical_type() {
         if *key != key_type {
             polars_bail!(ComputeError: "DictionaryArray must be initialized with a DataType::Dictionary whose integer is compatible to its keys")
         }
@@ -138,7 +139,7 @@ impl<K: DictionaryKey> DictionaryArray<K> {
     /// * the `data_type`'s values's data_type is not equal with `values.data_type()`
     /// * any of the keys's values is not represented in `usize` or is `>= values.len()`
     pub fn try_new(
-        data_type: DataType,
+        data_type: ArrowDataType,
         keys: PrimitiveArray<K>,
         values: Box<dyn Array>,
     ) -> PolarsResult<Self> {
@@ -146,7 +147,7 @@ impl<K: DictionaryKey> DictionaryArray<K> {
 
         if keys.null_count() != keys.len() {
             if K::always_fits_usize() {
-                // safety: we just checked that conversion to `usize` always
+                // SAFETY: we just checked that conversion to `usize` always
                 // succeeds
                 unsafe { check_indexes_unchecked(keys.values(), values.len()) }?;
             } else {
@@ -178,10 +179,11 @@ impl<K: DictionaryKey> DictionaryArray<K> {
     /// * the `data_type`'s logical type is not a `DictionaryArray`
     /// * the `data_type`'s keys is not compatible with `keys`
     /// * the `data_type`'s values's data_type is not equal with `values.data_type()`
+    ///
     /// # Safety
     /// The caller must ensure that every keys's values is represented in `usize` and is `< values.len()`
     pub unsafe fn try_new_unchecked(
-        data_type: DataType,
+        data_type: ArrowDataType,
         keys: PrimitiveArray<K>,
         values: Box<dyn Array>,
     ) -> PolarsResult<Self> {
@@ -195,7 +197,7 @@ impl<K: DictionaryKey> DictionaryArray<K> {
     }
 
     /// Returns a new empty [`DictionaryArray`].
-    pub fn new_empty(data_type: DataType) -> Self {
+    pub fn new_empty(data_type: ArrowDataType) -> Self {
         let values = Self::try_get_child(&data_type).unwrap();
         let values = new_empty_array(values.clone());
         Self::try_new(
@@ -208,7 +210,7 @@ impl<K: DictionaryKey> DictionaryArray<K> {
 
     /// Returns an [`DictionaryArray`] whose all elements are null
     #[inline]
-    pub fn new_null(data_type: DataType, length: usize) -> Self {
+    pub fn new_null(data_type: ArrowDataType, length: usize) -> Self {
         let values = Self::try_get_child(&data_type).unwrap();
         let values = new_null_array(values.clone(), 1);
         Self::try_new(
@@ -235,7 +237,7 @@ impl<K: DictionaryKey> DictionaryArray<K> {
         DictionaryValuesIter::new(self)
     }
 
-    /// Returns an iterator over the the values [`V::IterValue`].
+    /// Returns an iterator over the values [`V::IterValue`].
     ///
     /// # Panics
     ///
@@ -249,7 +251,7 @@ impl<K: DictionaryKey> DictionaryArray<K> {
         Ok(unsafe { DictionaryValuesIterTyped::new(keys, values) })
     }
 
-    /// Returns an iterator over the the optional values of  [`Option<V::IterValue>`].
+    /// Returns an iterator over the optional values of  [`Option<V::IterValue>`].
     ///
     /// # Panics
     ///
@@ -265,9 +267,9 @@ impl<K: DictionaryKey> DictionaryArray<K> {
         Ok(ZipValidity::new_with_validity(values_iter, self.validity()))
     }
 
-    /// Returns the [`DataType`] of this [`DictionaryArray`]
+    /// Returns the [`ArrowDataType`] of this [`DictionaryArray`]
     #[inline]
-    pub fn data_type(&self) -> &DataType {
+    pub fn data_type(&self) -> &ArrowDataType {
         &self.data_type
     }
 
@@ -275,13 +277,13 @@ impl<K: DictionaryKey> DictionaryArray<K> {
     #[inline]
     pub fn is_ordered(&self) -> bool {
         match self.data_type.to_logical_type() {
-            DataType::Dictionary(_, _, is_ordered) => *is_ordered,
+            ArrowDataType::Dictionary(_, _, is_ordered) => *is_ordered,
             _ => unreachable!(),
         }
     }
 
-    pub(crate) fn default_data_type(values_datatype: DataType) -> DataType {
-        DataType::Dictionary(K::KEY_TYPE, Box::new(values_datatype), false)
+    pub(crate) fn default_data_type(values_datatype: ArrowDataType) -> ArrowDataType {
+        ArrowDataType::Dictionary(K::KEY_TYPE, Box::new(values_datatype), false)
     }
 
     /// Slices this [`DictionaryArray`].
@@ -292,6 +294,7 @@ impl<K: DictionaryKey> DictionaryArray<K> {
     }
 
     /// Slices this [`DictionaryArray`].
+    ///
     /// # Safety
     /// Safe iff `offset + length <= self.len()`.
     pub unsafe fn slice_unchecked(&mut self, offset: usize, length: usize) {
@@ -340,14 +343,14 @@ impl<K: DictionaryKey> DictionaryArray<K> {
     /// Returns an iterator of the keys' values of the [`DictionaryArray`] as `usize`
     #[inline]
     pub fn keys_values_iter(&self) -> impl TrustedLen<Item = usize> + Clone + '_ {
-        // safety - invariant of the struct
+        // SAFETY: invariant of the struct
         self.keys.values_iter().map(|x| unsafe { x.as_usize() })
     }
 
     /// Returns an iterator of the keys' of the [`DictionaryArray`] as `usize`
     #[inline]
     pub fn keys_iter(&self) -> impl TrustedLen<Item = Option<usize>> + Clone + '_ {
-        // safety - invariant of the struct
+        // SAFETY: invariant of the struct
         self.keys.iter().map(|x| x.map(|x| unsafe { x.as_usize() }))
     }
 
@@ -356,7 +359,7 @@ impl<K: DictionaryKey> DictionaryArray<K> {
     /// This function panics iff `index >= self.len()`
     #[inline]
     pub fn key_value(&self, index: usize) -> usize {
-        // safety - invariant of the struct
+        // SAFETY: invariant of the struct
         unsafe { self.keys.values()[index].as_usize() }
     }
 
@@ -374,14 +377,14 @@ impl<K: DictionaryKey> DictionaryArray<K> {
     /// This function panics iff `index >= self.len()`
     #[inline]
     pub fn value(&self, index: usize) -> Box<dyn Scalar> {
-        // safety - invariant of this struct
+        // SAFETY: invariant of this struct
         let index = unsafe { self.keys.value(index).as_usize() };
         new_scalar(self.values.as_ref(), index)
     }
 
-    pub(crate) fn try_get_child(data_type: &DataType) -> PolarsResult<&DataType> {
+    pub(crate) fn try_get_child(data_type: &ArrowDataType) -> PolarsResult<&ArrowDataType> {
         Ok(match data_type.to_logical_type() {
-            DataType::Dictionary(_, values, _) => values.as_ref(),
+            ArrowDataType::Dictionary(_, values, _) => values.as_ref(),
             _ => {
                 polars_bail!(ComputeError: "Dictionaries must be initialized with DataType::Dictionary")
             },
