@@ -5,6 +5,8 @@ use polars_utils::unwrap::UnwrapUncheckedRelease;
 use super::*;
 #[cfg(feature = "dtype-struct")]
 use crate::chunked_array::builder::AnonymousOwnedListBuilder;
+use crate::chunked_array::builder::ListNullChunkedBuilder;
+use crate::series::implementations::null::NullChunked;
 
 pub trait AggList {
     /// # Safety
@@ -61,7 +63,7 @@ where
                     groups.iter().for_each(|(_, idx)| {
                         for i in idx.as_slice() {
                             if !old_validity.get_bit_unchecked(*i as usize) {
-                                validity.set_bit_unchecked(count, false)
+                                validity.set_unchecked(count, false);
                             }
                             count += 1;
                         }
@@ -124,7 +126,7 @@ where
                     groups.iter().for_each(|[first, len]| {
                         for i in *first..(*first + *len) {
                             if !old_validity.get_bit_unchecked(i as usize) {
-                                validity.set_bit_unchecked(count, false)
+                                validity.set_unchecked(count, false)
                             }
                             count += 1;
                         }
@@ -156,6 +158,27 @@ where
     }
 }
 
+impl AggList for NullChunked {
+    unsafe fn agg_list(&self, groups: &GroupsProxy) -> Series {
+        match groups {
+            GroupsProxy::Idx(groups) => {
+                let mut builder = ListNullChunkedBuilder::new(self.name(), groups.len());
+                for idx in groups.all().iter() {
+                    builder.append_with_len(idx.len());
+                }
+                builder.finish().into_series()
+            },
+            GroupsProxy::Slice { groups, .. } => {
+                let mut builder = ListNullChunkedBuilder::new(self.name(), groups.len());
+                for [_, len] in groups {
+                    builder.append_with_len(*len as usize);
+                }
+                builder.finish().into_series()
+            },
+        }
+    }
+}
+
 impl AggList for BooleanChunked {
     unsafe fn agg_list(&self, groups: &GroupsProxy) -> Series {
         match groups {
