@@ -54,7 +54,7 @@ use std::fs::File;
 use std::io::Write;
 use std::path::PathBuf;
 
-pub use parser::count_rows;
+pub use parser::{count_rows, CsvParserOptions};
 use polars_core::prelude::*;
 #[cfg(feature = "temporal")]
 use polars_time::prelude::*;
@@ -63,7 +63,7 @@ use rayon::prelude::*;
 pub use read::{CommentPrefix, CsvEncoding, CsvReader, NullValues};
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
-pub use write::{BatchedWriter, CsvWriter, QuoteStyle};
+pub use write::{BatchedWriter, CsvWriter, CsvWriterOptions, QuoteStyle};
 pub use write_impl::SerializeOptions;
 
 use crate::csv::read_impl::CoreReader;
