@@ -106,6 +106,7 @@ pub(crate) struct CoreReader<'a> {
     sample_size: usize,
     chunk_size: usize,
     low_memory: bool,
+    decimal_comma: bool,
     comment_prefix: Option<CommentPrefix>,
     quote_char: Option<u8>,
     eol_char: u8,
@@ -159,7 +160,9 @@ impl<'a> CoreReader<'a> {
         try_parse_dates: bool,
         raise_if_empty: bool,
         truncate_ragged_lines: bool,
+        decimal_comma: bool,
     ) -> PolarsResult<CoreReader<'a>> {
+        check_decimal_comma(decimal_comma, separator.unwrap_or(b','))?;
         #[cfg(any(feature = "decompress", feature = "decompress-fast"))]
         let mut reader_bytes = reader_bytes;
 
@@ -174,43 +177,41 @@ impl<'a> CoreReader<'a> {
         // check if schema should be inferred
         let separator = separator.unwrap_or(b',');
 
+        // We keep track of the inferred schema bool
+        // In case the file is compressed this schema inference is wrong and has to be done
+        // again after decompression.
+        #[cfg(any(feature = "decompress", feature = "decompress-fast"))]
+        {
+            let total_n_rows =
+                n_rows.map(|n| skip_rows + (has_header as usize) + skip_rows_after_header + n);
+            if let Some(b) =
+                decompress(&reader_bytes, total_n_rows, separator, quote_char, eol_char)
+            {
+                reader_bytes = ReaderBytes::Owned(b);
+            }
+        }
+
         let mut schema = match schema {
             Some(schema) => schema,
             None => {
-                {
-                    // We keep track of the inferred schema bool
-                    // In case the file is compressed this schema inference is wrong and has to be done
-                    // again after decompression.
-                    #[cfg(any(feature = "decompress", feature = "decompress-fast"))]
-                    {
-                        let total_n_rows = n_rows.map(|n| {
-                            skip_rows + (has_header as usize) + skip_rows_after_header + n
-                        });
-                        if let Some(b) =
-                            decompress(&reader_bytes, total_n_rows, separator, quote_char, eol_char)
-                        {
-                            reader_bytes = ReaderBytes::Owned(b);
-                        }
-                    }
-
-                    let (inferred_schema, _, _) = infer_file_schema(
-                        &reader_bytes,
-                        separator,
-                        max_records,
-                        has_header,
-                        schema_overwrite.as_deref(),
-                        &mut skip_rows,
-                        skip_rows_after_header,
-                        comment_prefix.as_ref(),
-                        quote_char,
-                        eol_char,
-                        null_values.as_ref(),
-                        try_parse_dates,
-                        raise_if_empty,
-                        &mut n_threads,
-                    )?;
-                    Arc::new(inferred_schema)
-                }
+                let (inferred_schema, _, _) = infer_file_schema(
+                    &reader_bytes,
+                    separator,
+                    max_records,
+                    has_header,
+                    schema_overwrite.as_deref(),
+                    &mut skip_rows,
+                    skip_rows_after_header,
+                    comment_prefix.as_ref(),
+                    quote_char,
+                    eol_char,
+                    null_values.as_ref(),
+                    try_parse_dates,
+                    raise_if_empty,
+                    &mut n_threads,
+                    decimal_comma,
+                )?;
+                Arc::new(inferred_schema)
             },
         };
         if let Some(dtypes) = dtype_overwrite {
@@ -263,6 +264,7 @@ impl<'a> CoreReader<'a> {
             to_cast,
             row_index,
             truncate_ragged_lines,
+            decimal_comma,
         })
     }
 
@@ -508,6 +510,7 @@ impl<'a> CoreReader<'a> {
                                 schema,
                                 self.quote_char,
                                 self.encoding,
+                                self.decimal_comma,
                             )?;
 
                             let local_bytes = &bytes[read..stop_at_nbytes];
@@ -592,6 +595,7 @@ impl<'a> CoreReader<'a> {
                             usize::MAX,
                             stop_at_nbytes,
                             starting_point_offset,
+                            self.decimal_comma,
                         )?;
 
                         cast_columns(&mut df, &self.to_cast, false, self.ignore_errors)?;
@@ -615,6 +619,7 @@ impl<'a> CoreReader<'a> {
                                 self.schema.as_ref(),
                                 self.quote_char,
                                 self.encoding,
+                                self.decimal_comma,
                             )?;
 
                             parse_lines(
@@ -697,9 +702,22 @@ fn read_chunk(
     chunk_size: usize,
     stop_at_nbytes: usize,
     starting_point_offset: Option<usize>,
+    decimal_comma: bool,
 ) -> PolarsResult<DataFrame> {
     let mut read = bytes_offset_thread;
-    let mut buffers = init_buffers(projection, capacity, schema, quote_char, encoding)?;
+    // There's an off-by-one error somewhere in the reading code, where it reads
+    // one more item than the requested capacity. Given the batch sizes are
+    // approximate (sometimes they're smaller), this isn't broken, but it does
+    // mean a bunch of extra allocation and copying. So we allocate a
+    // larger-by-one buffer so the size is more likely to be accurate.
+    let mut buffers = init_buffers(
+        projection,
+        capacity + 1,
+        schema,
+        quote_char,
+        encoding,
+        decimal_comma,
+    )?;
 
     let mut last_read = usize::MAX;
     loop {
