@@ -19,7 +19,7 @@ use crate::csv::CsvEncoding;
 use crate::mmap::ReaderBytes;
 use crate::prelude::parser::is_comment_line;
 use crate::prelude::{CommentPrefix, NullValues};
-use crate::utils::{BOOLEAN_RE, FLOAT_RE, INTEGER_RE};
+use crate::utils::{BOOLEAN_RE, FLOAT_RE, FLOAT_RE_DECIMAL, INTEGER_RE};
 
 pub(crate) fn get_file_chunks(
     bytes: &[u8],
@@ -60,7 +60,7 @@ pub(crate) fn get_file_chunks(
 }
 
 /// Infer the data type of a record
-fn infer_field_schema(string: &str, try_parse_dates: bool) -> DataType {
+fn infer_field_schema(string: &str, try_parse_dates: bool, decimal_comma: bool) -> DataType {
     // when quoting is enabled in the reader, these quotes aren't escaped, we default to
     // String for them
     if string.starts_with('"') {
@@ -91,7 +91,9 @@ fn infer_field_schema(string: &str, try_parse_dates: bool) -> DataType {
     // match regex in a particular order
     else if BOOLEAN_RE.is_match(string) {
         DataType::Boolean
-    } else if FLOAT_RE.is_match(string) {
+    } else if !decimal_comma && FLOAT_RE.is_match(string)
+        || decimal_comma && FLOAT_RE_DECIMAL.is_match(string)
+    {
         DataType::Float64
     } else if INTEGER_RE.is_match(string) {
         DataType::Int64
@@ -152,6 +154,7 @@ pub fn infer_file_schema_inner(
     recursion_count: u8,
     raise_if_empty: bool,
     n_threads: &mut Option<usize>,
+    decimal_comma: bool,
 ) -> PolarsResult<(Schema, usize, usize)> {
     // keep track so that we can determine the amount of bytes read
     let start_ptr = reader_bytes.as_ptr() as usize;
@@ -252,6 +255,7 @@ pub fn infer_file_schema_inner(
             recursion_count + 1,
             raise_if_empty,
             n_threads,
+            decimal_comma,
         );
     } else if !raise_if_empty {
         return Ok((Schema::new(), 0, 0));
@@ -329,17 +333,17 @@ pub fn infer_file_schema_inner(
                     };
                     let s = parse_bytes_with_encoding(slice_escaped, encoding)?;
                     let dtype = match &null_values {
-                        None => Some(infer_field_schema(&s, try_parse_dates)),
+                        None => Some(infer_field_schema(&s, try_parse_dates, decimal_comma)),
                         Some(NullValues::AllColumns(names)) => {
                             if !names.iter().any(|nv| nv == s.as_ref()) {
-                                Some(infer_field_schema(&s, try_parse_dates))
+                                Some(infer_field_schema(&s, try_parse_dates, decimal_comma))
                             } else {
                                 None
                             }
                         },
                         Some(NullValues::AllColumnsSingle(name)) => {
                             if s.as_ref() != name {
-                                Some(infer_field_schema(&s, try_parse_dates))
+                                Some(infer_field_schema(&s, try_parse_dates, decimal_comma))
                             } else {
                                 None
                             }
@@ -352,12 +356,12 @@ pub fn infer_file_schema_inner(
 
                             if let Some(null_name) = null_name {
                                 if null_name.1 != s.as_ref() {
-                                    Some(infer_field_schema(&s, try_parse_dates))
+                                    Some(infer_field_schema(&s, try_parse_dates, decimal_comma))
                                 } else {
                                     None
                                 }
                             } else {
-                                Some(infer_field_schema(&s, try_parse_dates))
+                                Some(infer_field_schema(&s, try_parse_dates, decimal_comma))
                             }
                         },
                     };
@@ -455,12 +459,20 @@ pub fn infer_file_schema_inner(
             recursion_count + 1,
             raise_if_empty,
             n_threads,
+            decimal_comma,
         );
     }
 
     Ok((Schema::from_iter(fields), rows_count, end_ptr - start_ptr))
 }
 
+pub(super) fn check_decimal_comma(decimal_comma: bool, separator: u8) -> PolarsResult<()> {
+    if decimal_comma {
+        polars_ensure!(b',' != separator, InvalidOperation: "'decimal_comma' argument cannot be combined with ',' quote char")
+    }
+    Ok(())
+}
+
 /// Infer the schema of a CSV file by reading through the first n rows of the file,
 /// with `max_read_rows` controlling the maximum number of rows to read.
 ///
@@ -488,7 +500,9 @@ pub fn infer_file_schema(
     try_parse_dates: bool,
     raise_if_empty: bool,
     n_threads: &mut Option<usize>,
+    decimal_comma: bool,
 ) -> PolarsResult<(Schema, usize, usize)> {
+    check_decimal_comma(decimal_comma, separator)?;
     infer_file_schema_inner(
         reader_bytes,
         separator,
@@ -505,6 +519,7 @@ pub fn infer_file_schema(
         0,
         raise_if_empty,
         n_threads,
+        decimal_comma,
     )
 }
 
