@@ -1,5 +1,6 @@
 use std::sync::Mutex;
 
+use arrow::array::ValueSize;
 use arrow::legacy::utils::CustomIterTools;
 use polars_core::prelude::*;
 use polars_plan::constants::MAP_LIST_NAME;
@@ -60,7 +61,7 @@ fn run_per_sublist(
             .par_iter()
             .map(|opt_s| {
                 opt_s.and_then(|s| {
-                    let df = DataFrame::new_no_checks(vec![s]);
+                    let df = s.into_frame();
                     let out = phys_expr.evaluate(&df, &state);
                     match out {
                         Ok(s) => Some(s),
@@ -75,7 +76,7 @@ fn run_per_sublist(
         err = m_err.into_inner().unwrap();
         ca
     } else {
-        let mut df_container = DataFrame::new_no_checks(vec![]);
+        let mut df_container = DataFrame::empty();
 
         lst.into_iter()
             .map(|s| {
@@ -119,10 +120,11 @@ fn run_on_group_by_engine(
     // List elements in a series.
     let values = Series::try_from(("", arr.values().clone())).unwrap();
     let inner_dtype = lst.inner_dtype();
-    // Ensure we use the logical type.
-    let values = values.cast(&inner_dtype).unwrap();
+    // SAFETY:
+    // Invariant in List means values physicals can be cast to inner dtype
+    let values = unsafe { values.cast_unchecked(&inner_dtype).unwrap() };
 
-    let df_context = DataFrame::new_no_checks(vec![values]);
+    let df_context = values.into_frame();
     let phys_expr = prepare_expression_for_context("", expr, &inner_dtype, Context::Aggregation)?;
 
     let state = ExecutionState::new();
@@ -148,7 +150,7 @@ pub trait ListNameSpaceExtension: IntoListNameSpace + Sized {
                 match e {
                     #[cfg(feature = "dtype-categorical")]
                     Expr::Cast {
-                        data_type: DataType::Categorical(_),
+                        data_type: DataType::Categorical(_, _) | DataType::Enum(_, _),
                         ..
                     } => {
                         polars_bail!(
@@ -174,7 +176,7 @@ pub trait ListNameSpaceExtension: IntoListNameSpace + Sized {
                 return Ok(Some(Series::new_empty(s.name(), output_field.data_type())));
             }
             if lst.null_count() == lst.len() {
-                return Ok(Some(s));
+                return Ok(Some(s.cast(output_field.data_type())?));
             }
 
             let fits_idx_size = lst.get_values_size() <= (IdxSize::MAX as usize);
