@@ -3,8 +3,6 @@ use polars_core::utils::{accumulate_dataframes_vertical, split_df};
 use rayon::prelude::*;
 
 use super::*;
-#[cfg(feature = "streaming")]
-use crate::physical_plan::planner::create_physical_plan;
 
 /// Take an input Executor and a multiple expressions
 pub struct PartitionGroupByExec {
@@ -69,7 +67,7 @@ fn run_partitions(
     state: &ExecutionState,
     n_threads: usize,
     maintain_order: bool,
-) -> PolarsResult<Vec<DataFrame>> {
+) -> PolarsResult<(Vec<DataFrame>, Vec<Vec<Series>>)> {
     // We do a partitioned group_by.
     // Meaning that we first do the group_by operation arbitrarily
     // split on several threads. Than the final result we apply the same group_by again.
@@ -77,11 +75,15 @@ fn run_partitions(
 
     let phys_aggs = &exec.phys_aggs;
     let keys = &exec.phys_keys;
+
+    let mut keys = DataFrame::from_iter(compute_keys(keys, df, state)?);
+    let splitted_keys = split_df(&mut keys, n_threads)?;
+
     POOL.install(|| {
         dfs.into_par_iter()
-            .map(|df| {
-                let keys = compute_keys(keys, &df, state)?;
-                let gb = df.group_by_with_series(keys, false, maintain_order)?;
+            .zip(splitted_keys)
+            .map(|(df, keys)| {
+                let gb = df.group_by_with_series(keys.into(), false, maintain_order)?;
                 let groups = gb.get_groups();
 
                 let mut columns = gb.keys();
@@ -106,7 +108,8 @@ fn run_partitions(
 
                 columns.extend_from_slice(&agg_columns);
 
-                DataFrame::new(columns)
+                let df = DataFrame::new(columns)?;
+                Ok((df, gb.keys()))
             })
             .collect()
     })
@@ -244,60 +247,12 @@ fn can_run_partitioned(
 }
 
 impl PartitionGroupByExec {
-    #[cfg(feature = "streaming")]
-    fn run_streaming(
-        &mut self,
-        state: &mut ExecutionState,
-        original_df: DataFrame,
-    ) -> Option<PolarsResult<DataFrame>> {
-        #[allow(clippy::needless_update)]
-        let group_by_options = GroupbyOptions {
-            slice: self.slice,
-            ..Default::default()
-        }
-        .into();
-        let lp = LogicalPlan::Aggregate {
-            input: Box::new(original_df.lazy().logical_plan),
-            keys: Arc::new(std::mem::take(&mut self.keys)),
-            aggs: std::mem::take(&mut self.aggs),
-            schema: self.output_schema.clone(),
-            apply: None,
-            maintain_order: false,
-            options: group_by_options,
-        };
-        let mut expr_arena = Default::default();
-        let mut lp_arena = Default::default();
-        let node = to_alp(lp, &mut expr_arena, &mut lp_arena).unwrap();
-
-        let inserted = streaming::insert_streaming_nodes(
-            node,
-            &mut lp_arena,
-            &mut expr_arena,
-            &mut vec![],
-            false,
-            false,
-            true,
-        )
-        .unwrap();
-
-        if inserted {
-            let mut phys_plan = create_physical_plan(node, &mut lp_arena, &mut expr_arena).unwrap();
-
-            if state.verbose() {
-                eprintln!("run STREAMING HASH AGGREGATION")
-            }
-            Some(phys_plan.execute(state))
-        } else {
-            None
-        }
-    }
-
     fn execute_impl(
         &mut self,
         state: &mut ExecutionState,
         mut original_df: DataFrame,
     ) -> PolarsResult<DataFrame> {
-        let dfs = {
+        let (splitted_dfs, splitted_keys) = {
             // already get the keys. This is the very last minute decision which group_by method we choose.
             // If the column is a categorical, we know the number of groups we have and can decide to continue
             // partitioned or go for the standard group_by. The partitioned is likely to be faster on a small number
@@ -316,13 +271,6 @@ impl PartitionGroupByExec {
                 );
             }
 
-            #[cfg(feature = "streaming")]
-            if !self.maintain_order && std::env::var("POLARS_NO_STREAMING_GROUPBY").is_err() {
-                if let Some(out) = self.run_streaming(state, original_df.clone()) {
-                    return out;
-                }
-            }
-
             if state.verbose() {
                 eprintln!("run PARTITIONED HASH AGGREGATION")
             }
@@ -339,12 +287,23 @@ impl PartitionGroupByExec {
             )?
         };
 
-        state.set_schema(self.output_schema.clone());
         // MERGE phase
-        // merge and hash aggregate again
-        let df = accumulate_dataframes_vertical(dfs)?;
+
+        let df = accumulate_dataframes_vertical(splitted_dfs)?;
+        let keys = splitted_keys
+            .into_iter()
+            .reduce(|mut acc, e| {
+                acc.iter_mut().zip(e).for_each(|(acc, e)| {
+                    let _ = acc.append(&e);
+                });
+                acc
+            })
+            .unwrap();
+
         // the partitioned group_by has added columns so we must update the schema.
-        let keys = self.keys(&df, state)?;
+        state.set_schema(self.output_schema.clone());
+
+        // merge and hash aggregate again
 
         // first get mutable access and optionally sort
         let gb = df.group_by_with_series(keys, true, self.maintain_order)?;
@@ -389,7 +348,7 @@ impl Executor for PartitionGroupByExec {
         #[cfg(debug_assertions)]
         {
             if state.verbose() {
-                println!("run PartitionGroupbyExec")
+                eprintln!("run PartitionGroupbyExec")
             }
         }
         let original_df = self.input.execute(state)?;
