@@ -23,6 +23,7 @@ use std::fmt::{Display, Formatter};
 pub(crate) use aggregation::*;
 pub(crate) use alias::*;
 pub(crate) use apply::*;
+use arrow::array::ArrayRef;
 use arrow::legacy::utils::CustomIterTools;
 pub(crate) use binary::*;
 pub(crate) use cast::*;
@@ -30,7 +31,6 @@ pub(crate) use column::*;
 pub(crate) use count::*;
 pub(crate) use filter::*;
 pub(crate) use literal::*;
-use polars_core::frame::group_by::GroupsProxy;
 use polars_core::prelude::*;
 use polars_io::predicates::PhysicalIoExpr;
 #[cfg(feature = "dynamic_group_by")]
@@ -97,7 +97,7 @@ impl AggState {
 
 // lazy update strategy
 #[cfg_attr(debug_assertions, derive(Debug))]
-#[derive(PartialEq)]
+#[derive(PartialEq, Clone, Copy)]
 pub(crate) enum UpdateGroups {
     /// don't update groups
     No,
@@ -132,6 +132,14 @@ pub struct AggregationContext<'a> {
 }
 
 impl<'a> AggregationContext<'a> {
+    pub(crate) fn dtype(&self) -> DataType {
+        match &self.state {
+            AggState::Literal(s) => s.dtype().clone(),
+            AggState::AggregatedList(s) => s.list().unwrap().inner_dtype(),
+            AggState::AggregatedScalar(s) => s.dtype().clone(),
+            AggState::NotAggregated(s) => s.dtype().clone(),
+        }
+    }
     pub(crate) fn groups(&mut self) -> &Cow<'a, GroupsProxy> {
         match self.update_groups {
             UpdateGroups::No => {},
@@ -231,7 +239,6 @@ impl<'a> AggregationContext<'a> {
 
     fn with_agg_state(&mut self, agg_state: AggState) {
         self.state = agg_state;
-        self.update_groups = UpdateGroups::No
     }
 
     fn from_agg_state(agg_state: AggState, groups: Cow<'a, GroupsProxy>) -> AggregationContext<'a> {
@@ -335,7 +342,7 @@ impl<'a> AggregationContext<'a> {
             AggState::NotAggregated(s) => {
                 // We should not aggregate literals!!
                 if self.state.safe_to_agg(&self.groups) {
-                    // safety:
+                    // SAFETY:
                     // groups are in bounds
                     let agg = unsafe { s.agg_list(&self.groups) };
                     self.update_groups = UpdateGroups::WithGroupsLen;
@@ -437,7 +444,7 @@ impl<'a> AggregationContext<'a> {
                     }
                 }
 
-                // safety:
+                // SAFETY:
                 // groups are in bounds
                 let out = unsafe { s.agg_list(&self.groups) };
                 self.state = AggState::AggregatedList(out.clone());
@@ -609,14 +616,9 @@ pub trait PhysicalExpr: Send + Sync {
     /// Can take &dyn Statistics and determine of a file should be
     /// read -> `true`
     /// or not -> `false`
-    #[cfg(feature = "parquet")]
     fn as_stats_evaluator(&self) -> Option<&dyn polars_io::predicates::StatsEvaluator> {
         None
     }
-
-    //
-    fn is_valid_aggregation(&self) -> bool;
-
     fn is_literal(&self) -> bool {
         false
     }
@@ -640,7 +642,7 @@ pub struct PhysicalIoHelper {
 }
 
 impl PhysicalIoExpr for PhysicalIoHelper {
-    fn evaluate(&self, df: &DataFrame) -> PolarsResult<Series> {
+    fn evaluate_io(&self, df: &DataFrame) -> PolarsResult<Series> {
         let mut state: ExecutionState = Default::default();
         if self.has_window_function {
             state.insert_has_window_function_flag();
