@@ -1,15 +1,17 @@
 use polars_utils::hashing::{hash_to_partition, DirtyHash};
+use polars_utils::nulls::IsNull;
 use polars_utils::total_ord::{ToTotalOrd, TotalEq, TotalHash};
 
 use super::*;
 
 /// Only keeps track of membership in right table
-pub(super) fn create_probe_table_semi_anti<T, I>(
+pub(super) fn build_table_semi_anti<T, I>(
     keys: Vec<I>,
+    join_nulls: bool,
 ) -> Vec<PlHashSet<<T as ToTotalOrd>::TotalOrdItem>>
 where
     T: TotalHash + TotalEq + DirtyHash + ToTotalOrd,
-    <T as ToTotalOrd>::TotalOrdItem: Send + Sync + Hash + Eq + DirtyHash,
+    <T as ToTotalOrd>::TotalOrdItem: Send + Sync + Hash + Eq + DirtyHash + IsNull,
     I: IntoIterator<Item = T> + Copy + Send + Sync,
 {
     let n_partitions = _set_partition_size();
@@ -22,7 +24,9 @@ where
         for keys in &keys {
             keys.into_iter().for_each(|k| {
                 let k = k.to_total_ord();
-                if partition_no == hash_to_partition(k.dirty_hash(), n_partitions) {
+                if partition_no == hash_to_partition(k.dirty_hash(), n_partitions)
+                    && (!k.is_null() || join_nulls)
+                {
                     hash_tbl.insert(k);
                 }
             });
@@ -37,14 +41,15 @@ where
 fn semi_anti_impl<T, I>(
     probe: Vec<I>,
     build: Vec<I>,
+    join_nulls: bool,
 ) -> impl ParallelIterator<Item = (IdxSize, bool)>
 where
     I: IntoIterator<Item = T> + Copy + Send + Sync,
     T: TotalHash + TotalEq + DirtyHash + ToTotalOrd,
-    <T as ToTotalOrd>::TotalOrdItem: Send + Sync + Hash + Eq + DirtyHash,
+    <T as ToTotalOrd>::TotalOrdItem: Send + Sync + Hash + Eq + DirtyHash + IsNull,
 {
     // first we hash one relation
-    let hash_sets = create_probe_table_semi_anti(build);
+    let hash_sets = build_table_semi_anti(build, join_nulls);
 
     // we determine the offset so that we later know which index to store in the join tuples
     let offsets = probe_to_offsets(&probe);
@@ -87,25 +92,33 @@ where
         })
 }
 
-pub(super) fn hash_join_tuples_left_anti<T, I>(probe: Vec<I>, build: Vec<I>) -> Vec<IdxSize>
+pub(super) fn hash_join_tuples_left_anti<T, I>(
+    probe: Vec<I>,
+    build: Vec<I>,
+    join_nulls: bool,
+) -> Vec<IdxSize>
 where
     I: IntoIterator<Item = T> + Copy + Send + Sync,
     T: TotalHash + TotalEq + DirtyHash + ToTotalOrd,
-    <T as ToTotalOrd>::TotalOrdItem: Send + Sync + Hash + Eq + DirtyHash,
+    <T as ToTotalOrd>::TotalOrdItem: Send + Sync + Hash + Eq + DirtyHash + IsNull,
 {
-    let par_iter = semi_anti_impl(probe, build)
+    let par_iter = semi_anti_impl(probe, build, join_nulls)
         .filter(|tpls| !tpls.1)
         .map(|tpls| tpls.0);
     POOL.install(|| par_iter.collect())
 }
 
-pub(super) fn hash_join_tuples_left_semi<T, I>(probe: Vec<I>, build: Vec<I>) -> Vec<IdxSize>
+pub(super) fn hash_join_tuples_left_semi<T, I>(
+    probe: Vec<I>,
+    build: Vec<I>,
+    join_nulls: bool,
+) -> Vec<IdxSize>
 where
     I: IntoIterator<Item = T> + Copy + Send + Sync,
     T: TotalHash + TotalEq + DirtyHash + ToTotalOrd,
-    <T as ToTotalOrd>::TotalOrdItem: Send + Sync + Hash + Eq + DirtyHash,
+    <T as ToTotalOrd>::TotalOrdItem: Send + Sync + Hash + Eq + DirtyHash + IsNull,
 {
-    let par_iter = semi_anti_impl(probe, build)
+    let par_iter = semi_anti_impl(probe, build, join_nulls)
         .filter(|tpls| tpls.1)
         .map(|tpls| tpls.0);
     POOL.install(|| par_iter.collect())
