@@ -3,8 +3,8 @@ use std::task::Poll;
 
 use ahash::AHashMap;
 use arrow::array::Array;
-use arrow::chunk::Chunk;
 use arrow::datatypes::ArrowSchema;
+use arrow::record_batch::RecordBatch;
 use futures::future::BoxFuture;
 use futures::{AsyncWrite, AsyncWriteExt, FutureExt, Sink, TryFutureExt};
 use polars_error::{polars_bail, to_compute_err, PolarsError, PolarsResult};
@@ -14,7 +14,7 @@ use super::{Encoding, SchemaDescriptor, WriteOptions};
 use crate::parquet::metadata::KeyValue;
 use crate::parquet::write::{FileStreamer, WriteOptions as ParquetWriteOptions};
 
-/// Sink that writes array [`chunks`](Chunk) as a Parquet file.
+/// Sink that writes array [`chunks`](RecordBatch) as a Parquet file.
 ///
 /// Any values in the sink's `metadata` field will be written to the file's footer
 /// when the sink is closed.
@@ -110,13 +110,16 @@ where
     }
 }
 
-impl<'a, W> Sink<Chunk<Box<dyn Array>>> for FileSink<'a, W>
+impl<'a, W> Sink<RecordBatch<Box<dyn Array>>> for FileSink<'a, W>
 where
     W: AsyncWrite + Send + Unpin + 'a,
 {
     type Error = PolarsError;
 
-    fn start_send(self: Pin<&mut Self>, item: Chunk<Box<dyn Array>>) -> Result<(), Self::Error> {
+    fn start_send(
+        self: Pin<&mut Self>,
+        item: RecordBatch<Box<dyn Array>>,
+    ) -> Result<(), Self::Error> {
         if self.schema.fields.len() != item.arrays().len() {
             polars_bail!(InvalidOperation:
                 "The number of arrays in the chunk must equal the number of fields in the schema"
