@@ -7,6 +7,7 @@ use polars_core::with_match_physical_integer_polars_type;
 #[cfg(feature = "parquet")]
 use polars_io::predicates::{PhysicalIoExpr, StatsEvaluator};
 use polars_ops::prelude::JoinType;
+use polars_plan::prelude::expr_ir::{ExprIR, OutputName};
 use polars_plan::prelude::*;
 
 use crate::executors::operators::{HstackOperator, PlaceHolder};
@@ -17,28 +18,28 @@ use crate::executors::{operators, sources};
 use crate::expressions::PhysicalPipedExpr;
 use crate::operators::{Operator, Sink as SinkTrait, Source};
 use crate::pipeline::dispatcher::ThreadedSink;
-use crate::pipeline::PipeLine;
+use crate::pipeline::{PhysOperator, PipeLine};
 
 pub type CallBacks = PlHashMap<Node, PlaceHolder>;
 
 fn exprs_to_physical<F>(
-    exprs: &[Node],
+    exprs: &[ExprIR],
     expr_arena: &Arena<AExpr>,
     to_physical: &F,
     schema: Option<&SchemaRef>,
 ) -> PolarsResult<Vec<Arc<dyn PhysicalPipedExpr>>>
 where
-    F: Fn(Node, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
+    F: Fn(&ExprIR, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
 {
     exprs
         .iter()
-        .map(|node| to_physical(*node, expr_arena, schema))
+        .map(|e| to_physical(e, expr_arena, schema))
         .collect()
 }
 
 #[allow(unused_variables)]
 fn get_source<F>(
-    source: ALogicalPlan,
+    source: IR,
     operator_objects: &mut Vec<Box<dyn Operator>>,
     expr_arena: &Arena<AExpr>,
     to_physical: &F,
@@ -46,9 +47,9 @@ fn get_source<F>(
     verbose: bool,
 ) -> PolarsResult<Box<dyn Source>>
 where
-    F: Fn(Node, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
+    F: Fn(&ExprIR, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
 {
-    use ALogicalPlan::*;
+    use IR::*;
     match source {
         DataFrameScan {
             df,
@@ -60,7 +61,7 @@ where
             let mut df = (*df).clone();
             if push_predicate {
                 if let Some(predicate) = selection {
-                    let predicate = to_physical(predicate, expr_arena, output_schema.as_ref())?;
+                    let predicate = to_physical(&predicate, expr_arena, output_schema.as_ref())?;
                     let op = operators::FilterOperator { predicate };
                     let op = Box::new(op) as Box<dyn Operator>;
                     operator_objects.push(op)
@@ -87,10 +88,11 @@ where
             #[cfg(not(feature = "parquet"))]
             let is_parquet = false;
 
-            if let (false, true, Some(predicate)) = (is_parquet, push_predicate, predicate) {
+            if let (false, true, Some(predicate)) = (is_parquet, push_predicate, predicate.clone())
+            {
                 #[cfg(feature = "parquet")]
                 debug_assert!(!matches!(scan_type, FileScan::Parquet { .. }));
-                let predicate = to_physical(predicate, expr_arena, output_schema.as_ref())?;
+                let predicate = to_physical(&predicate, expr_arena, output_schema.as_ref())?;
                 let op = operators::FilterOperator { predicate };
                 let op = Box::new(op) as Box<dyn Operator>;
                 operator_objects.push(op)
@@ -117,6 +119,7 @@ where
                     metadata,
                 } => {
                     let predicate = predicate
+                        .as_ref()
                         .map(|predicate| {
                             let p = to_physical(predicate, expr_arena, output_schema.as_ref())?;
                             // Arc's all the way down. :(
@@ -158,15 +161,15 @@ where
 
 pub fn get_sink<F>(
     node: Node,
-    lp_arena: &Arena<ALogicalPlan>,
+    lp_arena: &Arena<IR>,
     expr_arena: &mut Arena<AExpr>,
     to_physical: &F,
     callbacks: &mut CallBacks,
 ) -> PolarsResult<Box<dyn SinkTrait>>
 where
-    F: Fn(Node, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
+    F: Fn(&ExprIR, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
 {
-    use ALogicalPlan::*;
+    use IR::*;
     let out = match lp_arena.get(node) {
         Sink { input, payload } => {
             let input_schema = lp_arena.get(*input).schema(lp_arena);
@@ -338,28 +341,32 @@ where
         Sort {
             input,
             by_column,
-            args,
+            slice,
+            sort_options,
         } => {
             let input_schema = lp_arena.get(*input).schema(lp_arena).into_owned();
 
             if by_column.len() == 1 {
-                let by_column = aexpr_to_leaf_names_iter(by_column[0], expr_arena)
+                let by_column = aexpr_to_leaf_names_iter(by_column[0].node(), expr_arena)
                     .next()
                     .unwrap();
                 let index = input_schema.try_index_of(by_column.as_ref())?;
 
-                let sort_sink = SortSink::new(index, args.clone(), input_schema);
+                let sort_sink = SortSink::new(index, *slice, sort_options.clone(), input_schema);
                 Box::new(sort_sink) as Box<dyn SinkTrait>
             } else {
                 let sort_idx = by_column
                     .iter()
-                    .map(|node| {
-                        let name = aexpr_to_leaf_names_iter(*node, expr_arena).next().unwrap();
+                    .map(|e| {
+                        let name = aexpr_to_leaf_names_iter(e.node(), expr_arena)
+                            .next()
+                            .unwrap();
                         input_schema.try_index_of(name.as_ref())
                     })
                     .collect::<PolarsResult<Vec<_>>>()?;
 
-                let sort_sink = SortSinkMultiple::new(args.clone(), input_schema, sort_idx)?;
+                let sort_sink =
+                    SortSinkMultiple::new(*slice, sort_options.clone(), input_schema, sort_idx)?;
                 Box::new(sort_sink) as Box<dyn SinkTrait>
             }
         },
@@ -371,7 +378,11 @@ where
                 None => {
                     let keys = input_schema
                         .iter_names()
-                        .map(|name| expr_arena.add(AExpr::Column(Arc::from(name.as_str()))))
+                        .map(|name| {
+                            let name: Arc<str> = Arc::from(name.as_str());
+                            let node = expr_arena.add(AExpr::Column(name.clone()));
+                            ExprIR::new(node, OutputName::Alias(name))
+                        })
                         .collect::<Vec<_>>();
                     let aggs = vec![];
                     (keys, aggs, input_schema.clone())
@@ -384,9 +395,11 @@ where
                         .map(|key| {
                             let (_, name, dtype) = input_schema.get_full(key.as_str()).unwrap();
                             group_by_out_schema.with_column(name.clone(), dtype.clone());
-                            expr_arena.add(AExpr::Column(Arc::from(key.as_str())))
+                            let name: Arc<str> = Arc::from(key.as_str());
+                            let node = expr_arena.add(AExpr::Column(name.clone()));
+                            ExprIR::new(node, OutputName::Alias(name))
                         })
-                        .collect();
+                        .collect::<Vec<_>>();
 
                     let aggs = input_schema
                         .iter_names()
@@ -397,8 +410,10 @@ where
                                 let (_, name, dtype) =
                                     input_schema.get_full(name.as_str()).unwrap();
                                 group_by_out_schema.with_column(name.clone(), dtype.clone());
-                                let col = expr_arena.add(AExpr::Column(Arc::from(name.as_str())));
-                                Some(match options.keep_strategy {
+
+                                let name: Arc<str> = Arc::from(name.as_str());
+                                let col = expr_arena.add(AExpr::Column(name.clone()));
+                                let node = match options.keep_strategy {
                                     UniqueKeepStrategy::First | UniqueKeepStrategy::Any => {
                                         expr_arena.add(AExpr::Agg(AAggExpr::First(col)))
                                     },
@@ -408,7 +423,8 @@ where
                                     UniqueKeepStrategy::None => {
                                         unreachable!()
                                     },
-                                })
+                                };
+                                Some(ExprIR::new(node, OutputName::Alias(name)))
                             }
                         })
                         .collect();
@@ -427,9 +443,9 @@ where
             let mut agg_fns = Vec::with_capacity(aggs.len());
             let mut input_agg_dtypes = Vec::with_capacity(aggs.len());
 
-            for node in &aggs {
+            for e in &aggs {
                 let (input_dtype, index, agg_fn) =
-                    convert_to_hash_agg(*node, expr_arena, &input_schema, &to_physical);
+                    convert_to_hash_agg(e.node(), expr_arena, &input_schema, &to_physical);
                 aggregation_columns.push(index);
                 agg_fns.push(agg_fn);
                 input_agg_dtypes.push(input_dtype);
@@ -447,7 +463,7 @@ where
 
             Box::new(ReProjectSink::new(input_schema, group_by_sink))
         },
-        Aggregate {
+        GroupBy {
             input,
             keys,
             aggs,
@@ -467,9 +483,9 @@ where
             let mut agg_fns = Vec::with_capacity(aggs.len());
             let mut input_agg_dtypes = Vec::with_capacity(aggs.len());
 
-            for node in aggs {
+            for e in aggs {
                 let (input_dtype, index, agg_fn) =
-                    convert_to_hash_agg(*node, expr_arena, &input_schema, &to_physical);
+                    convert_to_hash_agg(e.node(), expr_arena, &input_schema, &to_physical);
                 aggregation_columns.push(index);
                 agg_fns.push(agg_fn);
                 input_agg_dtypes.push(input_dtype);
@@ -533,7 +549,7 @@ pub fn get_dummy_operator() -> PlaceHolder {
 }
 
 fn get_hstack<F>(
-    exprs: &[Node],
+    exprs: &[ExprIR],
     expr_arena: &Arena<AExpr>,
     to_physical: &F,
     input_schema: SchemaRef,
@@ -541,7 +557,7 @@ fn get_hstack<F>(
     unchecked: bool,
 ) -> PolarsResult<HstackOperator>
 where
-    F: Fn(Node, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
+    F: Fn(&ExprIR, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
 {
     Ok(operators::HstackOperator {
         exprs: exprs_to_physical(exprs, expr_arena, &to_physical, Some(&input_schema))?,
@@ -553,16 +569,22 @@ where
 
 pub fn get_operator<F>(
     node: Node,
-    lp_arena: &Arena<ALogicalPlan>,
+    lp_arena: &Arena<IR>,
     expr_arena: &Arena<AExpr>,
     to_physical: &F,
 ) -> PolarsResult<Box<dyn Operator>>
 where
-    F: Fn(Node, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
+    F: Fn(&ExprIR, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
 {
-    use ALogicalPlan::*;
+    use IR::*;
     let op = match lp_arena.get(node) {
-        Projection { expr, input, .. } => {
+        SimpleProjection { input, columns, .. } => {
+            let input_schema = lp_arena.get(*input).schema(lp_arena);
+            let columns = columns.iter_names().cloned().collect();
+            let op = operators::SimpleProjectionOperator::new(columns, input_schema.into_owned());
+            Box::new(op) as Box<dyn Operator>
+        },
+        Select { expr, input, .. } => {
             let input_schema = lp_arena.get(*input).schema(lp_arena);
 
             let cse_exprs = expr.cse_exprs();
@@ -617,21 +639,12 @@ where
 
             Box::new(op) as Box<dyn Operator>
         },
-        Selection { predicate, input } => {
+        Filter { predicate, input } => {
             let input_schema = lp_arena.get(*input).schema(lp_arena);
-            let predicate = to_physical(*predicate, expr_arena, Some(input_schema.as_ref()))?;
+            let predicate = to_physical(predicate, expr_arena, Some(input_schema.as_ref()))?;
             let op = operators::FilterOperator { predicate };
             Box::new(op) as Box<dyn Operator>
         },
-        MapFunction {
-            function: FunctionNode::FastProjection { columns, .. },
-            input,
-        } => {
-            let input_schema = lp_arena.get(*input).schema(lp_arena);
-            let op =
-                operators::FastProjectionOperator::new(columns.clone(), input_schema.into_owned());
-            Box::new(op) as Box<dyn Operator>
-        },
         MapFunction { function, .. } => {
             let op = operators::FunctionOperator::new(function.clone());
             Box::new(op) as Box<dyn Operator>
@@ -653,7 +666,7 @@ pub fn create_pipeline<F>(
     sources: &[Node],
     operators: Vec<Box<dyn Operator>>,
     sink_nodes: Vec<(usize, Node, Rc<RefCell<u32>>)>,
-    lp_arena: &Arena<ALogicalPlan>,
+    lp_arena: &Arena<IR>,
     expr_arena: &mut Arena<AExpr>,
     to_physical: F,
     verbose: bool,
@@ -663,9 +676,9 @@ pub fn create_pipeline<F>(
     callbacks: &mut CallBacks,
 ) -> PolarsResult<PipeLine>
 where
-    F: Fn(Node, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
+    F: Fn(&ExprIR, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
 {
-    use ALogicalPlan::*;
+    use IR::*;
 
     let mut source_objects = Vec::with_capacity(sources.len());
     let mut operator_objects = Vec::with_capacity(operators.len() + 1);
@@ -745,7 +758,9 @@ where
 
     Ok(PipeLine::new(
         source_objects,
-        unsafe { std::mem::transmute(operator_objects) },
+        unsafe {
+            std::mem::transmute::<Vec<Box<dyn Operator>>, Vec<PhysOperator>>(operator_objects)
+        },
         sinks,
         verbose,
     ))
