@@ -4,6 +4,7 @@ use std::path::PathBuf;
 
 use polars_core::prelude::*;
 
+use crate::constants::UNLIMITED_CACHE;
 use crate::prelude::*;
 
 impl Expr {
@@ -70,7 +71,7 @@ pub struct DotNode<'a> {
     pub fmt: &'a str,
 }
 
-impl LogicalPlan {
+impl DslPlan {
     fn write_single_node(&self, acc_str: &mut String, node: DotNode) -> std::fmt::Result {
         let fmt_node = node.fmt.replace('"', r#"\""#);
         writeln!(acc_str, "graph  polars_query {{\n\"[{fmt_node}]\"")?;
@@ -127,7 +128,7 @@ impl LogicalPlan {
         prev_node: DotNode,
         id_map: &mut PlHashMap<String, String>,
     ) -> std::fmt::Result {
-        use LogicalPlan::*;
+        use DslPlan::*;
         let (mut branch, id) = id;
 
         match self {
@@ -160,23 +161,25 @@ impl LogicalPlan {
             Cache {
                 input,
                 id: cache_id,
-                count,
+                cache_hits,
             } => {
-                let fmt = if *count == usize::MAX {
+                // Always increment cache ids as the `DotNode[0, 0]` will insert a new graph, which we don't want.
+                let cache_id = cache_id.saturating_add(1);
+                let fmt = if *cache_hits == UNLIMITED_CACHE {
                     Cow::Borrowed("CACHE")
                 } else {
-                    Cow::Owned(format!("CACHE: {} times", *count))
+                    Cow::Owned(format!("CACHE: {} times", *cache_hits))
                 };
                 let current_node = DotNode {
-                    branch: *cache_id,
-                    id: *cache_id,
+                    branch: cache_id,
+                    id: cache_id,
                     fmt: &fmt,
                 };
                 // here we take the cache id, to ensure the same cached subplans get the same ids
                 self.write_dot(acc_str, prev_node, current_node, id_map)?;
-                input.dot(acc_str, (*cache_id, cache_id + 1), current_node, id_map)
+                input.dot(acc_str, (cache_id, cache_id + 1), current_node, id_map)
             },
-            Selection { predicate, input } => {
+            Filter { predicate, input } => {
                 let pred = fmt_predicate(Some(predicate));
                 let fmt = format!("FILTER BY {pred}");
 
@@ -196,14 +199,14 @@ impl LogicalPlan {
                 "PYTHON",
                 &[],
                 options.with_columns.as_ref().map(|s| s.as_slice()),
-                options.schema.len(),
+                Some(options.schema.len()),
                 &options.predicate,
                 branch,
                 id,
                 id_map,
             ),
-            Projection { expr, input, .. } => {
-                let schema = input.schema().map_err(|_| {
+            Select { expr, input, .. } => {
+                let schema = input.compute_schema().map_err(|_| {
                     eprintln!("could not determine schema");
                     std::fmt::Error
                 })?;
@@ -230,7 +233,7 @@ impl LogicalPlan {
                 self.write_dot(acc_str, prev_node, current_node, id_map)?;
                 input.dot(acc_str, (branch, id + 1), current_node, id_map)
             },
-            Aggregate {
+            GroupBy {
                 input, keys, aggs, ..
             } => {
                 let mut s_keys = String::with_capacity(128);
@@ -339,7 +342,7 @@ impl LogicalPlan {
                     name,
                     paths.as_ref(),
                     options.with_columns.as_ref().map(|cols| cols.as_slice()),
-                    file_info.schema.len(),
+                    file_info.as_ref().map(|fi| fi.schema.len()),
                     predicate,
                     branch,
                     id,
@@ -404,15 +407,6 @@ impl LogicalPlan {
                 self.write_dot(acc_str, prev_node, current_node, id_map)?;
                 input.dot(acc_str, (branch, id + 1), current_node, id_map)
             },
-            Error { err, .. } => {
-                let fmt = format!("{:?}", &err.0);
-                let current_node = DotNode {
-                    branch,
-                    id,
-                    fmt: &fmt,
-                };
-                self.write_dot(acc_str, prev_node, current_node, id_map)
-            },
         }
     }
 
@@ -424,7 +418,7 @@ impl LogicalPlan {
         name: &str,
         path: &[PathBuf],
         with_columns: Option<&[String]>,
-        total_columns: usize,
+        total_columns: Option<usize>,
         predicate: &Option<P>,
         branch: usize,
         id: usize,
@@ -446,6 +440,9 @@ impl LogicalPlan {
         };
 
         let pred = fmt_predicate(predicate.as_ref());
+        let total_columns = total_columns
+            .map(|v| format!("{v}"))
+            .unwrap_or_else(|| "?".to_string());
         let fmt = format!(
             "{name} SCAN {};\nπ {}/{};\nσ {}",
             path_fmt, n_columns_fmt, total_columns, pred,
