@@ -27,6 +27,11 @@ impl FunctionExpr {
 
             // Other expressions
             Boolean(func) => func.get_field(mapper),
+            #[cfg(feature = "business")]
+            Business(func) => match func {
+                BusinessFunction::BusinessDayCount { .. } => mapper.with_dtype(DataType::Int32),
+                BusinessFunction::AddBusinessDay { .. } => mapper.with_same_dtype(),
+            },
             #[cfg(feature = "abs")]
             Abs => mapper.with_same_dtype(),
             Negate => mapper.with_same_dtype(),
@@ -251,7 +256,7 @@ impl FunctionExpr {
             #[cfg(feature = "rle")]
             RLE => mapper.map_dtype(|dt| {
                 DataType::Struct(vec![
-                    Field::new("lengths", DataType::UInt64),
+                    Field::new("lengths", DataType::Int32),
                     Field::new("values", dt.clone()),
                 ])
             }),
@@ -281,6 +286,8 @@ impl FunctionExpr {
             MeanHorizontal => mapper.map_to_float_dtype(),
             #[cfg(feature = "ewma")]
             EwmMean { .. } => mapper.map_to_float_dtype(),
+            #[cfg(feature = "ewma_by")]
+            EwmMeanBy { .. } => mapper.map_to_float_dtype(),
             #[cfg(feature = "ewma")]
             EwmStd { .. } => mapper.map_to_float_dtype(),
             #[cfg(feature = "ewma")]
@@ -301,6 +308,14 @@ impl FunctionExpr {
             ExtendConstant => mapper.with_same_dtype(),
         }
     }
+
+    pub(crate) fn output_name(&self) -> Option<&ColumnName> {
+        match self {
+            #[cfg(feature = "dtype-struct")]
+            FunctionExpr::StructExpr(StructFunction::FieldByName(name)) => Some(name),
+            _ => None,
+        }
+    }
 }
 
 pub struct FieldsMapper<'a> {
@@ -466,14 +481,16 @@ impl<'a> FieldsMapper<'a> {
     }
 
     pub(super) fn pow_dtype(&self) -> PolarsResult<Field> {
-        // base, exponent
-        match (self.fields[0].data_type(), self.fields[1].data_type()) {
-            (
-                base_dtype,
-                DataType::UInt8 | DataType::UInt16 | DataType::UInt32 | DataType::UInt64,
-            ) => Ok(Field::new(self.fields[0].name(), base_dtype.clone())),
-            (DataType::Float32, _) => Ok(Field::new(self.fields[0].name(), DataType::Float32)),
-            (_, _) => Ok(Field::new(self.fields[0].name(), DataType::Float64)),
+        let base_dtype = self.fields[0].data_type();
+        let exponent_dtype = self.fields[1].data_type();
+        if base_dtype.is_integer() {
+            if exponent_dtype.is_float() {
+                Ok(Field::new(self.fields[0].name(), exponent_dtype.clone()))
+            } else {
+                Ok(Field::new(self.fields[0].name(), base_dtype.clone()))
+            }
+        } else {
+            Ok(Field::new(self.fields[0].name(), base_dtype.clone()))
         }
     }
 
