@@ -1,5 +1,8 @@
 use std::hash::{Hash, Hasher};
 
+use polars_utils::arena::{Arena, Node};
+
+use crate::logical_plan::ArenaExprIter;
 use crate::prelude::AExpr;
 
 impl Hash for AExpr {
@@ -22,7 +25,7 @@ impl Hash for AExpr {
                 options.hash(state);
             },
             AExpr::Agg(agg) => agg.hash(state),
-            AExpr::SortBy { descending, .. } => descending.hash(state),
+            AExpr::SortBy { sort_options, .. } => sort_options.hash(state),
             AExpr::Cast { strict, .. } => strict.hash(state),
             AExpr::Window { options, .. } => options.hash(state),
             AExpr::BinaryExpr { op, .. } => op.hash(state),
@@ -30,3 +33,13 @@ impl Hash for AExpr {
         }
     }
 }
+
+pub(crate) fn traverse_and_hash_aexpr<H: Hasher>(
+    node: Node,
+    expr_arena: &Arena<AExpr>,
+    state: &mut H,
+) {
+    for (_, ae) in expr_arena.iter(node) {
+        ae.hash(state);
+    }
+}
