@@ -12,10 +12,10 @@ mod semi_anti_join;
 use polars_core::datatypes::PlHashSet;
 use polars_core::prelude::*;
 use polars_io::RowIndex;
+use recursive::recursive;
 #[cfg(feature = "semi_anti_join")]
 use semi_anti_join::process_semi_anti_join;
 
-use crate::logical_plan::Context;
 use crate::prelude::optimizer::projection_pushdown::generic::process_generic;
 use crate::prelude::optimizer::projection_pushdown::group_by::process_group_by;
 use crate::prelude::optimizer::projection_pushdown::hconcat::process_hconcat;
@@ -24,11 +24,9 @@ use crate::prelude::optimizer::projection_pushdown::joins::process_join;
 use crate::prelude::optimizer::projection_pushdown::projection::process_projection;
 use crate::prelude::optimizer::projection_pushdown::rename::process_rename;
 use crate::prelude::*;
-use crate::utils::{
-    aexpr_assign_renamed_leaf, aexpr_to_leaf_names, check_input_node, expr_is_projected_upstream,
-};
+use crate::utils::aexpr_to_leaf_names;
 
-fn init_vec() -> Vec<Node> {
+fn init_vec() -> Vec<ColumnNode> {
     Vec::with_capacity(16)
 }
 fn init_set() -> PlHashSet<Arc<str>> {
@@ -37,7 +35,7 @@ fn init_set() -> PlHashSet<Arc<str>> {
 
 /// utility function to get names of the columns needed in projection at scan level
 fn get_scan_columns(
-    acc_projections: &mut Vec<Node>,
+    acc_projections: &mut Vec<ColumnNode>,
     expr_arena: &Arena<AExpr>,
     row_index: Option<&RowIndex>,
 ) -> Option<Arc<Vec<String>>> {
@@ -45,17 +43,16 @@ fn get_scan_columns(
     if !acc_projections.is_empty() {
         let mut columns = Vec::with_capacity(acc_projections.len());
         for expr in acc_projections {
-            for name in aexpr_to_leaf_names(*expr, expr_arena) {
-                // we shouldn't project the row-count column, as that is generated
-                // in the scan
-                let push = match row_index {
-                    Some(rc) if name.as_ref() != rc.name.as_str() => true,
-                    None => true,
-                    _ => false,
-                };
-                if push {
-                    columns.push((*name).to_owned())
-                }
+            let name = column_node_to_name(*expr, expr_arena);
+            // we shouldn't project the row-count column, as that is generated
+            // in the scan
+            let push = match row_index {
+                Some(rc) if name.as_ref() != rc.name.as_str() => true,
+                None => true,
+                _ => false,
+            };
+            if push {
+                columns.push((*name).to_owned())
             }
         }
         with_columns = Some(Arc::new(columns));
@@ -71,24 +68,23 @@ fn get_scan_columns(
 ///
 /// - `expands_schema`. An unnest adds more columns to a schema, so we cannot use fast path
 fn split_acc_projections(
-    acc_projections: Vec<Node>,
+    acc_projections: Vec<ColumnNode>,
     down_schema: &Schema,
     expr_arena: &Arena<AExpr>,
     expands_schema: bool,
-) -> (Vec<Node>, Vec<Node>, PlHashSet<Arc<str>>) {
+) -> (Vec<ColumnNode>, Vec<ColumnNode>, PlHashSet<Arc<str>>) {
     // If node above has as many columns as the projection there is nothing to pushdown.
     if !expands_schema && down_schema.len() == acc_projections.len() {
         let local_projections = acc_projections;
         (vec![], local_projections, PlHashSet::new())
     } else {
-        let (acc_projections, local_projections): (Vec<Node>, Vec<Node>) = acc_projections
+        let (acc_projections, local_projections): (Vec<_>, Vec<_>) = acc_projections
             .into_iter()
-            .partition(|expr| check_input_node(*expr, down_schema, expr_arena));
+            .partition(|expr| check_input_column_node(*expr, down_schema, expr_arena));
         let mut names = init_set();
         for proj in &acc_projections {
-            for name in aexpr_to_leaf_names(*proj, expr_arena) {
-                names.insert(name);
-            }
+            let name = column_node_to_name(*proj, expr_arena);
+            names.insert(name);
         }
         (acc_projections, local_projections, names)
     }
@@ -97,34 +93,33 @@ fn split_acc_projections(
 /// utility function such that we can recurse all binary expressions in the expression tree
 fn add_expr_to_accumulated(
     expr: Node,
-    acc_projections: &mut Vec<Node>,
+    acc_projections: &mut Vec<ColumnNode>,
     projected_names: &mut PlHashSet<Arc<str>>,
     expr_arena: &Arena<AExpr>,
 ) {
     for root_node in aexpr_to_column_nodes_iter(expr, expr_arena) {
-        for name in aexpr_to_leaf_names_iter(root_node, expr_arena) {
-            if projected_names.insert(name) {
-                acc_projections.push(root_node)
-            }
+        let name = column_node_to_name(root_node, expr_arena);
+        if projected_names.insert(name) {
+            acc_projections.push(root_node)
         }
     }
 }
 
 fn add_str_to_accumulated(
     name: &str,
-    acc_projections: &mut Vec<Node>,
+    acc_projections: &mut Vec<ColumnNode>,
     projected_names: &mut PlHashSet<Arc<str>>,
     expr_arena: &mut Arena<AExpr>,
 ) {
     // if empty: all columns are already projected.
     if !acc_projections.is_empty() && !projected_names.contains(name) {
-        let node = expr_arena.add(AExpr::Column(Arc::from(name)));
+        let node = expr_arena.add(AExpr::Column(ColumnName::from(name)));
         add_expr_to_accumulated(node, acc_projections, projected_names, expr_arena);
     }
 }
 
 fn update_scan_schema(
-    acc_projections: &[Node],
+    acc_projections: &[ColumnNode],
     expr_arena: &Arena<AExpr>,
     schema: &Schema,
     sort_projections: bool,
@@ -132,12 +127,9 @@ fn update_scan_schema(
     let mut new_schema = Schema::with_capacity(acc_projections.len());
     let mut new_cols = Vec::with_capacity(acc_projections.len());
     for node in acc_projections.iter() {
-        for name in aexpr_to_leaf_names_iter(*node, expr_arena) {
-            let item = schema.get_full(&name).ok_or_else(|| {
-                polars_err!(ComputeError: "column '{}' not available in 'DataFrame' with {:?}", name, schema)
-            })?;
-            new_cols.push(item);
-        }
+        let name = column_node_to_name(*node, expr_arena);
+        let item = schema.try_get_full(&name)?;
+        new_cols.push(item);
     }
     // make sure that the projections are sorted by the schema.
     if sort_projections {
@@ -163,12 +155,12 @@ impl ProjectionPushDown {
     /// Projection will be done at this node, but we continue optimization
     fn no_pushdown_restart_opt(
         &mut self,
-        lp: ALogicalPlan,
-        acc_projections: Vec<Node>,
+        lp: IR,
+        acc_projections: Vec<ColumnNode>,
         projections_seen: usize,
-        lp_arena: &mut Arena<ALogicalPlan>,
+        lp_arena: &mut Arena<IR>,
         expr_arena: &mut Arena<AExpr>,
-    ) -> PolarsResult<ALogicalPlan> {
+    ) -> PolarsResult<IR> {
         let inputs = lp.get_inputs();
         let exprs = lp.get_exprs();
 
@@ -190,15 +182,26 @@ impl ProjectionPushDown {
             .collect::<PolarsResult<Vec<_>>>()?;
         let lp = lp.with_exprs_and_input(exprs, new_inputs);
 
-        let builder = ALogicalPlanBuilder::from_lp(lp, expr_arena, lp_arena);
-        Ok(self.finish_node(acc_projections, builder))
+        let builder = IRBuilder::from_lp(lp, expr_arena, lp_arena);
+        Ok(self.finish_node_simple_projection(&acc_projections, builder))
     }
 
-    fn finish_node(
+    fn finish_node_simple_projection(
         &mut self,
-        local_projections: Vec<Node>,
-        builder: ALogicalPlanBuilder,
-    ) -> ALogicalPlan {
+        local_projections: &[ColumnNode],
+        builder: IRBuilder,
+    ) -> IR {
+        if !local_projections.is_empty() {
+            builder
+                .project_simple_nodes(local_projections.iter().map(|node| node.0))
+                .unwrap()
+                .build()
+        } else {
+            builder.build()
+        }
+    }
+
+    fn finish_node(&mut self, local_projections: Vec<ExprIR>, builder: IRBuilder) -> IR {
         if !local_projections.is_empty() {
             builder
                 .project(local_projections, Default::default())
@@ -213,34 +216,31 @@ impl ProjectionPushDown {
         &mut self,
         schema_left: &Schema,
         schema_right: &Schema,
-        proj: Node,
-        pushdown_left: &mut Vec<Node>,
-        pushdown_right: &mut Vec<Node>,
+        proj: ColumnNode,
+        pushdown_left: &mut Vec<ColumnNode>,
+        pushdown_right: &mut Vec<ColumnNode>,
         names_left: &mut PlHashSet<Arc<str>>,
         names_right: &mut PlHashSet<Arc<str>>,
         expr_arena: &Arena<AExpr>,
     ) -> (bool, bool) {
         let mut pushed_at_least_one = false;
         let mut already_projected = false;
-        let names = aexpr_to_leaf_names(proj, expr_arena);
-        let root_projections = aexpr_to_column_nodes(proj, expr_arena);
 
-        for (name, root_projection) in names.into_iter().zip(root_projections) {
-            let is_in_left = names_left.contains(&name);
-            let is_in_right = names_right.contains(&name);
-            already_projected |= is_in_left;
-            already_projected |= is_in_right;
+        let name = column_node_to_name(proj, expr_arena);
+        let is_in_left = names_left.contains(&name);
+        let is_in_right = names_right.contains(&name);
+        already_projected |= is_in_left;
+        already_projected |= is_in_right;
 
-            if check_input_node(root_projection, schema_left, expr_arena) && !is_in_left {
-                names_left.insert(name.clone());
-                pushdown_left.push(proj);
-                pushed_at_least_one = true;
-            }
-            if check_input_node(root_projection, schema_right, expr_arena) && !is_in_right {
-                names_right.insert(name.clone());
-                pushdown_right.push(proj);
-                pushed_at_least_one = true;
-            }
+        if check_input_column_node(proj, schema_left, expr_arena) && !is_in_left {
+            names_left.insert(name.clone());
+            pushdown_left.push(proj);
+            pushed_at_least_one = true;
+        }
+        if check_input_column_node(proj, schema_right, expr_arena) && !is_in_right {
+            names_right.insert(name.clone());
+            pushdown_right.push(proj);
+            pushed_at_least_one = true;
         }
 
         (pushed_at_least_one, already_projected)
@@ -250,10 +250,10 @@ impl ProjectionPushDown {
     fn pushdown_and_assign(
         &mut self,
         input: Node,
-        acc_projections: Vec<Node>,
+        acc_projections: Vec<ColumnNode>,
         names: PlHashSet<Arc<str>>,
         projections_seen: usize,
-        lp_arena: &mut Arena<ALogicalPlan>,
+        lp_arena: &mut Arena<IR>,
         expr_arena: &mut Arena<AExpr>,
     ) -> PolarsResult<()> {
         let alp = lp_arena.take(input);
@@ -277,13 +277,13 @@ impl ProjectionPushDown {
     fn pushdown_and_assign_check_schema(
         &mut self,
         input: Node,
-        acc_projections: Vec<Node>,
+        acc_projections: Vec<ColumnNode>,
         projections_seen: usize,
-        lp_arena: &mut Arena<ALogicalPlan>,
+        lp_arena: &mut Arena<IR>,
         expr_arena: &mut Arena<AExpr>,
         // an unnest changes/expands the schema
         expands_schema: bool,
-    ) -> PolarsResult<Vec<Node>> {
+    ) -> PolarsResult<Vec<ColumnNode>> {
         let alp = lp_arena.take(input);
         let down_schema = alp.schema(lp_arena);
 
@@ -306,26 +306,26 @@ impl ProjectionPushDown {
     ///
     /// # Arguments
     ///
-    /// * `AlogicalPlan` - Arena based logical plan tree representing the query.
+    /// * `IR` - Arena based logical plan tree representing the query.
     /// * `acc_projections` - The projections we accumulate during tree traversal.
     /// * `names` - We keep track of the names to ensure we don't do duplicate projections.
     /// * `projections_seen` - Count the number of projection operations during tree traversal.
     /// * `lp_arena` - The local memory arena for the logical plan.
     /// * `expr_arena` - The local memory arena for the expressions.
-    ///
+    #[recursive]
     fn push_down(
         &mut self,
-        logical_plan: ALogicalPlan,
-        mut acc_projections: Vec<Node>,
+        logical_plan: IR,
+        mut acc_projections: Vec<ColumnNode>,
         mut projected_names: PlHashSet<Arc<str>>,
         projections_seen: usize,
-        lp_arena: &mut Arena<ALogicalPlan>,
+        lp_arena: &mut Arena<IR>,
         expr_arena: &mut Arena<AExpr>,
-    ) -> PolarsResult<ALogicalPlan> {
-        use ALogicalPlan::*;
+    ) -> PolarsResult<IR> {
+        use IR::*;
 
         match logical_plan {
-            Projection { expr, input, .. } => process_projection(
+            Select { expr, input, .. } => process_projection(
                 self,
                 input,
                 expr.exprs(),
@@ -335,6 +335,19 @@ impl ProjectionPushDown {
                 lp_arena,
                 expr_arena,
             ),
+            SimpleProjection { columns, input, .. } => {
+                let exprs = names_to_expr_irs(columns.iter_names(), expr_arena);
+                process_projection(
+                    self,
+                    input,
+                    exprs,
+                    acc_projections,
+                    projected_names,
+                    projections_seen,
+                    lp_arena,
+                    expr_arena,
+                )
+            },
             DataFrameScan {
                 df,
                 schema,
@@ -438,21 +451,18 @@ impl ProjectionPushDown {
             Sort {
                 input,
                 by_column,
-                args,
+                slice,
+                sort_options,
             } => {
                 if !acc_projections.is_empty() {
                     // Make sure that the column(s) used for the sort is projected
                     by_column.iter().for_each(|node| {
-                        aexpr_to_column_nodes(*node, expr_arena)
-                            .iter()
-                            .for_each(|root| {
-                                add_expr_to_accumulated(
-                                    *root,
-                                    &mut acc_projections,
-                                    &mut projected_names,
-                                    expr_arena,
-                                );
-                            })
+                        add_expr_to_accumulated(
+                            node.node(),
+                            &mut acc_projections,
+                            &mut projected_names,
+                            expr_arena,
+                        );
                     });
                 }
 
@@ -467,7 +477,8 @@ impl ProjectionPushDown {
                 Ok(Sort {
                     input,
                     by_column,
-                    args,
+                    slice,
+                    sort_options,
                 })
             },
             Distinct { input, options } => {
@@ -506,11 +517,11 @@ impl ProjectionPushDown {
                 )?;
                 Ok(Distinct { input, options })
             },
-            Selection { predicate, input } => {
+            Filter { predicate, input } => {
                 if !acc_projections.is_empty() {
                     // make sure that the filter column is projected
                     add_expr_to_accumulated(
-                        predicate,
+                        predicate.node(),
                         &mut acc_projections,
                         &mut projected_names,
                         expr_arena,
@@ -524,9 +535,9 @@ impl ProjectionPushDown {
                     lp_arena,
                     expr_arena,
                 )?;
-                Ok(Selection { predicate, input })
+                Ok(Filter { predicate, input })
             },
-            Aggregate {
+            GroupBy {
                 input,
                 keys,
                 aggs,
@@ -555,7 +566,7 @@ impl ProjectionPushDown {
                 left_on,
                 right_on,
                 options,
-                ..
+                schema,
             } => match options.args.how {
                 #[cfg(feature = "semi_anti_join")]
                 JoinType::Semi | JoinType::Anti => process_semi_anti_join(
@@ -583,6 +594,7 @@ impl ProjectionPushDown {
                     projections_seen,
                     lp_arena,
                     expr_arena,
+                    &schema,
                 ),
             },
             HStack {
@@ -691,22 +703,22 @@ impl ProjectionPushDown {
                 if acc_projections.is_empty() {
                     Ok(logical_plan)
                 } else {
-                    Ok(
-                        ALogicalPlanBuilder::from_lp(logical_plan, expr_arena, lp_arena)
-                            .project(acc_projections, Default::default())
-                            .build(),
-                    )
+                    Ok(IRBuilder::from_lp(logical_plan, expr_arena, lp_arena)
+                        .project_simple_nodes(acc_projections)
+                        .unwrap()
+                        .build())
                 }
             },
+            Invalid => unreachable!(),
         }
     }
 
     pub fn optimize(
         &mut self,
-        logical_plan: ALogicalPlan,
-        lp_arena: &mut Arena<ALogicalPlan>,
+        logical_plan: IR,
+        lp_arena: &mut Arena<IR>,
         expr_arena: &mut Arena<AExpr>,
-    ) -> PolarsResult<ALogicalPlan> {
+    ) -> PolarsResult<IR> {
         let acc_projections = init_vec();
         let names = init_set();
         self.push_down(
