@@ -5,6 +5,7 @@ use polars_core::error::*;
 #[cfg(feature = "regex")]
 use regex::Regex;
 
+use crate::constants::LEN;
 use crate::logical_plan::visitor::{VisitRecursion, Visitor};
 use crate::prelude::visitor::AexprNode;
 use crate::prelude::*;
@@ -36,11 +37,16 @@ impl UpperExp for AExpr {
                 )
             },
             AExpr::Gather { .. } => "gather",
-            AExpr::SortBy { descending, .. } => {
+            AExpr::SortBy { sort_options, .. } => {
                 write!(f, "sort_by:")?;
-                for i in descending {
+                for i in &sort_options.descending {
                     write!(f, "{}", *i as u8)?;
                 }
+                write!(
+                    f,
+                    "{}{}",
+                    sort_options.nulls_last as u8, sort_options.multithreaded as u8
+                )?;
                 return Ok(());
             },
             AExpr::Filter { .. } => "filter",
@@ -56,7 +62,7 @@ impl UpperExp for AExpr {
             AExpr::Window { .. } => "window",
             AExpr::Wildcard => "*",
             AExpr::Slice { .. } => "slice",
-            AExpr::Len => "len",
+            AExpr::Len => LEN,
             AExpr::Nth(v) => return write!(f, "nth({})", v),
         };
 
@@ -66,7 +72,7 @@ impl UpperExp for AExpr {
 
 pub enum TreeFmtNode<'a> {
     Expression(Option<String>, &'a Expr),
-    LogicalPlan(Option<String>, &'a LogicalPlan),
+    LogicalPlan(Option<String>, &'a DslPlan),
 }
 
 struct TreeFmtNodeData<'a>(String, Vec<TreeFmtNode<'a>>);
@@ -86,7 +92,7 @@ fn multiline_expression(expr: &str) -> Cow<'_, str> {
 }
 
 impl<'a> TreeFmtNode<'a> {
-    pub fn root_logical_plan(lp: &'a LogicalPlan) -> Self {
+    pub fn root_logical_plan(lp: &'a DslPlan) -> Self {
         Self::LogicalPlan(None, lp)
     }
 
@@ -117,7 +123,7 @@ impl<'a> TreeFmtNode<'a> {
     }
 
     fn node_data(&self) -> TreeFmtNodeData<'_> {
-        use LogicalPlan::*;
+        use DslPlan::*;
         use TreeFmtNode::{Expression as NE, LogicalPlan as NL};
         use {with_header as wh, TreeFmtNodeData as ND};
 
@@ -180,18 +186,28 @@ impl<'a> TreeFmtNode<'a> {
                     .map(|(i, lp)| NL(Some(format!("PLAN {i}:")), lp))
                     .collect(),
             ),
-            NL(h, Cache { input, id, count }) => ND(
-                wh(h, &format!("CACHE[id: {:x}, count: {}]", *id, *count)),
+            NL(
+                h,
+                Cache {
+                    input,
+                    id,
+                    cache_hits,
+                },
+            ) => ND(
+                wh(
+                    h,
+                    &format!("CACHE[id: {:x}, cache_hits: {}]", *id, *cache_hits),
+                ),
                 vec![NL(None, input)],
             ),
-            NL(h, Selection { input, predicate }) => ND(
+            NL(h, Filter { input, predicate }) => ND(
                 wh(h, "FILTER"),
                 vec![
                     NE(Some("predicate:".to_string()), predicate),
                     NL(Some("FROM:".to_string()), input),
                 ],
             ),
-            NL(h, Projection { expr, input, .. }) => ND(
+            NL(h, Select { expr, input, .. }) => ND(
                 wh(h, "SELECT"),
                 expr.iter()
                     .map(|expr| NE(Some("expression:".to_string()), expr))
@@ -200,7 +216,7 @@ impl<'a> TreeFmtNode<'a> {
             ),
             NL(
                 h,
-                LogicalPlan::Sort {
+                DslPlan::Sort {
                     input, by_column, ..
                 },
             ) => ND(
@@ -213,7 +229,7 @@ impl<'a> TreeFmtNode<'a> {
             ),
             NL(
                 h,
-                Aggregate {
+                GroupBy {
                     input, keys, aggs, ..
                 },
             ) => ND(
@@ -269,14 +285,13 @@ impl<'a> TreeFmtNode<'a> {
                 ),
                 vec![NL(None, input)],
             ),
-            NL(h, LogicalPlan::Slice { input, offset, len }) => ND(
+            NL(h, DslPlan::Slice { input, offset, len }) => ND(
                 wh(h, &format!("SLICE[offset: {offset}, len: {len}]")),
                 vec![NL(None, input)],
             ),
             NL(h, MapFunction { input, function }) => {
                 ND(wh(h, &format!("{function}")), vec![NL(None, input)])
             },
-            NL(h, Error { input, err }) => ND(wh(h, &format!("{err:?}")), vec![NL(None, input)]),
             NL(h, ExtContext { input, .. }) => ND(wh(h, "EXTERNAL_CONTEXT"), vec![NL(None, input)]),
             NL(h, Sink { input, payload }) => ND(
                 wh(
@@ -304,10 +319,15 @@ pub(crate) struct TreeFmtVisitor {
 
 impl Visitor for TreeFmtVisitor {
     type Node = AexprNode;
+    type Arena = Arena<AExpr>;
 
     /// Invoked before any children of `node` are visited.
-    fn pre_visit(&mut self, node: &Self::Node) -> PolarsResult<VisitRecursion> {
-        let ae = node.to_aexpr();
+    fn pre_visit(
+        &mut self,
+        node: &Self::Node,
+        arena: &Self::Arena,
+    ) -> PolarsResult<VisitRecursion> {
+        let ae = node.to_aexpr(arena);
         let repr = format!("{:E}", ae);
 
         if self.levels.len() <= self.depth {
@@ -331,7 +351,11 @@ impl Visitor for TreeFmtVisitor {
         Ok(VisitRecursion::Continue)
     }
 
-    fn post_visit(&mut self, _node: &Self::Node) -> PolarsResult<VisitRecursion> {
+    fn post_visit(
+        &mut self,
+        _node: &Self::Node,
+        _arena: &Self::Arena,
+    ) -> PolarsResult<VisitRecursion> {
         // we finished this branch so we decrease in depth, back the caller node
         self.depth -= 1;
 
@@ -800,7 +824,8 @@ mod test {
 
         let mut visitor = TreeFmtVisitor::default();
 
-        AexprNode::with_context(node, &mut arena, |ae_node| ae_node.visit(&mut visitor)).unwrap();
+        let ae_node = AexprNode::new(node);
+        ae_node.visit(&mut visitor, &arena).unwrap();
         let expected: &[&[&str]] = &[
             &["sum"],
             &["binary: +"],
@@ -820,7 +845,7 @@ mod test {
 
         let mut visitor = TreeFmtVisitor::default();
 
-        AexprNode::with_context(node, &mut arena, |ae_node| ae_node.visit(&mut visitor)).unwrap();
+        AexprNode::new(node).visit(&mut visitor, &arena).unwrap();
 
         let expected_lines = vec![
             "            0            1               2                3            4",
@@ -869,8 +894,8 @@ mod test {
         let node = to_aexpr(e, &mut arena);
 
         let mut visitor = TreeFmtVisitor::default();
-
-        AexprNode::with_context(node, &mut arena, |ae_node| ae_node.visit(&mut visitor)).unwrap();
+        let ae_node = AexprNode::new(node);
+        ae_node.visit(&mut visitor, &arena).unwrap();
 
         let expected_lines = vec![
             "                 0                 1               2                3            4",
