@@ -1,6 +1,8 @@
 use std::io::Cursor;
+use std::num::NonZeroUsize;
 
 use polars::io::RowIndex;
+use polars_core::export::chrono;
 
 use super::*;
 
@@ -13,6 +15,7 @@ fn write_csv() {
 
     CsvWriter::new(&mut buf)
         .include_header(true)
+        .with_batch_size(NonZeroUsize::new(1).unwrap())
         .finish(&mut df)
         .expect("csv written");
     let csv = std::str::from_utf8(&buf).unwrap();
@@ -36,6 +39,91 @@ fn write_csv() {
     assert_eq!("0,22.1\r\n1,19.9\r\n2,7.0\r\n3,2.0\r\n4,3.0\r\n", csv);
 }
 
+#[test]
+fn write_dates() {
+    let s0 = Series::new("date", [chrono::NaiveDate::from_yo_opt(2024, 33), None]);
+    let s1 = Series::new("time", [None, chrono::NaiveTime::from_hms_opt(19, 50, 0)]);
+    let s2 = Series::new(
+        "datetime",
+        [
+            Some(chrono::NaiveDateTime::new(
+                chrono::NaiveDate::from_ymd_opt(2000, 12, 1).unwrap(),
+                chrono::NaiveTime::from_num_seconds_from_midnight_opt(99, 49575634).unwrap(),
+            )),
+            None,
+        ],
+    );
+    let mut df = DataFrame::new(vec![s0, s1, s2.clone()]).unwrap();
+
+    let mut buf: Vec<u8> = Vec::new();
+    CsvWriter::new(&mut buf)
+        .include_header(true)
+        .with_batch_size(NonZeroUsize::new(1).unwrap())
+        .finish(&mut df)
+        .expect("csv written");
+    let csv = std::str::from_utf8(&buf).unwrap();
+    assert_eq!(
+        "date,time,datetime\n2024-02-02,,2000-12-01T00:01:39.049\n,19:50:00.000000000,\n",
+        csv,
+    );
+
+    buf.clear();
+    CsvWriter::new(&mut buf)
+        .include_header(true)
+        .with_batch_size(NonZeroUsize::new(1).unwrap())
+        .with_date_format(Some("%d/%m/%Y".into()))
+        .with_time_format(Some("%H%M%S".into()))
+        .with_datetime_format(Some("%Y-%m-%d %H:%M:%S".into()))
+        .finish(&mut df)
+        .expect("csv written");
+    let csv = std::str::from_utf8(&buf).unwrap();
+    assert_eq!(
+        "date,time,datetime\n02/02/2024,,2000-12-01 00:01:39\n,195000,\n",
+        csv,
+    );
+
+    buf.clear();
+    CsvWriter::new(&mut buf)
+        .include_header(true)
+        .with_batch_size(NonZeroUsize::new(1).unwrap())
+        .with_date_format(Some("%<invalid format>".into()))
+        .finish(&mut df)
+        .expect_err("invalid date/time format should err");
+
+    buf.clear();
+    CsvWriter::new(&mut buf)
+        .include_header(true)
+        .with_batch_size(NonZeroUsize::new(1).unwrap())
+        .with_date_format(Some("%H".into()))
+        .finish(&mut df)
+        .expect_err("invalid date/time format should err");
+
+    buf.clear();
+    CsvWriter::new(&mut buf)
+        .include_header(true)
+        .with_batch_size(NonZeroUsize::new(1).unwrap())
+        .with_datetime_format(Some("%Z".into()))
+        .finish(&mut df)
+        .expect_err("invalid date/time format should err");
+
+    let with_timezone = polars_ops::chunked_array::replace_time_zone(
+        s2.slice(0, 1).datetime().unwrap(),
+        Some("America/New_York"),
+        &StringChunked::new("", ["raise"]),
+        NonExistent::Raise,
+    )
+    .unwrap()
+    .into_series();
+    let mut with_timezone_df = DataFrame::new(vec![with_timezone]).unwrap();
+    buf.clear();
+    CsvWriter::new(&mut buf)
+        .include_header(false)
+        .finish(&mut with_timezone_df)
+        .expect("csv written");
+    let csv = std::str::from_utf8(&buf).unwrap();
+    assert_eq!("2000-12-01T00:01:39.049-0500\n", csv);
+}
+
 #[test]
 fn test_read_csv_file() {
     let file = std::fs::File::open(FOODS_CSV).unwrap();
@@ -50,7 +138,7 @@ fn test_read_csv_file() {
 #[test]
 fn test_parser() -> PolarsResult<()> {
     let s = r#"
- "sepal.length","sepal.width","petal.length","petal.width","variety"
+ "sepal_length","sepal_width","petal_length","petal_width","variety"
  5.1,3.5,1.4,.2,"Setosa"
  4.9,3,1.4,.2,"Setosa"
  4.7,3.2,1.3,.2,"Setosa"
@@ -69,7 +157,7 @@ fn test_parser() -> PolarsResult<()> {
         .unwrap();
 
     let s = r#"
-         "sepal.length","sepal.width","petal.length","petal.width","variety"
+         "sepal_length","sepal_width","petal_length","petal_width","variety"
          5.1,3.5,1.4,.2,"Setosa"
          5.1,3.5,1.4,.2,"Setosa"
  "#;
@@ -85,7 +173,7 @@ fn test_parser() -> PolarsResult<()> {
         .finish()
         .unwrap();
 
-    let s = r#""sepal.length","sepal.width","petal.length","petal.width","variety"
+    let s = r#""sepal_length","sepal_width","petal_length","petal_width","variety"
         5.1,3.5,1.4,.2,"Setosa"
         4.9,3,1.4,.2,"Setosa"
         4.7,3.2,1.3,.2,"Setosa"
@@ -106,8 +194,8 @@ fn test_parser() -> PolarsResult<()> {
     assert_eq!(col.get(0)?, AnyValue::String("Setosa"));
     assert_eq!(col.get(2)?, AnyValue::String("Setosa"));
 
-    assert_eq!("sepal.length", df.get_columns()[0].name());
-    assert_eq!(1, df.column("sepal.length").unwrap().chunks().len());
+    assert_eq!("sepal_length", df.get_columns()[0].name());
+    assert_eq!(1, df.column("sepal_length").unwrap().chunks().len());
     assert_eq!(df.height(), 7);
 
     // test windows line endings
@@ -339,7 +427,7 @@ id090,id048,id0000067778,24,2,51862,4,9,
 
 #[test]
 fn test_new_line_escape() {
-    let s = r#""sepal.length","sepal.width","petal.length","petal.width","variety"
+    let s = r#""sepal_length","sepal_width","petal_length","petal_width","variety"
  5.1,3.5,1.4,.2,"Setosa
  texts after new line character"
  4.9,3,1.4,.2,"Setosa"
