@@ -52,7 +52,7 @@ fn test_groups_update_binary_shift_log() -> PolarsResult<()> {
     .lazy()
     .group_by([col("b")])
     .agg([col("a") - col("a").shift(lit(1)).log(2.0)])
-    .sort("b", Default::default())
+    .sort(["b"], Default::default())
     .explode([col("a")])
     .collect()?;
     assert_eq!(
