@@ -1,4 +1,5 @@
 use std::path::PathBuf;
+use std::sync::Mutex;
 
 use polars::io::mmap::MmapBytesReader;
 use polars::io::RowIndex;
@@ -17,8 +18,7 @@ enum BatchedReader {
 #[pyclass]
 #[repr(transparent)]
 pub struct PyBatchedCsv {
-    // option because we cannot get a self by value in pyo3
-    reader: BatchedReader,
+    reader: Mutex<BatchedReader>,
 }
 
 #[pymethods]
@@ -30,7 +30,7 @@ impl PyBatchedCsv {
         projection, separator, rechunk, columns, encoding, n_threads, path, overwrite_dtype,
         overwrite_dtype_slice, low_memory, comment_prefix, quote_char, null_values,
         missing_utf8_is_empty_string, try_parse_dates, skip_rows_after_header, row_index,
-        sample_size, eol_char, raise_if_empty, truncate_ragged_lines)
+        sample_size, eol_char, raise_if_empty, truncate_ragged_lines, decimal_comma)
     )]
     fn new(
         infer_schema_length: Option<usize>,
@@ -60,6 +60,7 @@ impl PyBatchedCsv {
         eol_char: &str,
         raise_if_empty: bool,
         truncate_ragged_lines: bool,
+        decimal_comma: bool,
     ) -> PyResult<PyBatchedCsv> {
         let null_values = null_values.map(|w| w.0);
         let eol_char = eol_char.as_bytes()[0];
@@ -118,6 +119,7 @@ impl PyBatchedCsv {
             .with_row_index(row_index)
             .sample_size(sample_size)
             .truncate_ragged_lines(truncate_ragged_lines)
+            .with_decimal_comma(decimal_comma)
             .raise_if_empty(raise_if_empty);
 
         let reader = if low_memory {
@@ -132,15 +134,23 @@ impl PyBatchedCsv {
             BatchedReader::MMap(reader)
         };
 
-        Ok(PyBatchedCsv { reader })
+        Ok(PyBatchedCsv {
+            reader: Mutex::new(reader),
+        })
     }
 
-    fn next_batches(&mut self, n: usize) -> PyResult<Option<Vec<PyDataFrame>>> {
-        let batches = match &mut self.reader {
-            BatchedReader::MMap(reader) => reader.next_batches(n),
-            BatchedReader::Read(reader) => reader.next_batches(n),
-        }
-        .map_err(PyPolarsErr::from)?;
+    fn next_batches(&self, py: Python, n: usize) -> PyResult<Option<Vec<PyDataFrame>>> {
+        let reader = &self.reader;
+        let batches = py.allow_threads(move || {
+            let reader = &mut *reader
+                .lock()
+                .map_err(|e| PyPolarsErr::Other(e.to_string()))?;
+            match reader {
+                BatchedReader::MMap(reader) => reader.next_batches(n),
+                BatchedReader::Read(reader) => reader.next_batches(n),
+            }
+            .map_err(PyPolarsErr::from)
+        })?;
 
         // SAFETY: same memory layout
         let batches = unsafe {
