@@ -33,20 +33,12 @@ use crate::series::PySeries;
 use crate::{PyDataFrame, PyLazyFrame};
 
 pub(crate) fn slice_to_wrapped<T>(slice: &[T]) -> &[Wrap<T>] {
-    // SAFETY:
-    // Wrap is transparent.
-    unsafe { std::mem::transmute(slice) }
-}
-
-pub(crate) fn slice_extract_wrapped<T>(slice: &[Wrap<T>]) -> &[T] {
-    // SAFETY:
-    // Wrap is transparent.
+    // SAFETY: Wrap is transparent.
     unsafe { std::mem::transmute(slice) }
 }
 
 pub(crate) fn vec_extract_wrapped<T>(buf: Vec<Wrap<T>>) -> Vec<T> {
-    // SAFETY:
-    // Wrap is transparent.
+    // SAFETY: Wrap is transparent.
     unsafe { std::mem::transmute(buf) }
 }
 
@@ -78,13 +70,13 @@ pub(crate) fn get_lf(obj: &PyAny) -> PyResult<LazyFrame> {
     Ok(pydf.extract::<PyLazyFrame>()?.ldf)
 }
 
-pub(crate) fn get_series(obj: &PyAny) -> PyResult<Series> {
+pub(crate) fn get_series(obj: &Bound<'_, PyAny>) -> PyResult<Series> {
     let pydf = obj.getattr(intern!(obj.py(), "_s"))?;
     Ok(pydf.extract::<PySeries>()?.series)
 }
 
 pub(crate) fn to_series(py: Python, s: PySeries) -> PyObject {
-    let series = SERIES.as_ref(py);
+    let series = SERIES.bind(py);
     let constructor = series
         .getattr(intern!(series.py(), "_from_pyseries"))
         .unwrap();
@@ -93,7 +85,7 @@ pub(crate) fn to_series(py: Python, s: PySeries) -> PyObject {
 
 #[cfg(feature = "csv")]
 impl<'a> FromPyObject<'a> for Wrap<NullValues> {
-    fn extract(ob: &'a PyAny) -> PyResult<Self> {
+    fn extract_bound(ob: &Bound<'a, PyAny>) -> PyResult<Self> {
         if let Ok(s) = ob.extract::<String>() {
             Ok(Wrap(NullValues::AllColumnsSingle(s)))
         } else if let Ok(s) = ob.extract::<Vec<String>>() {
@@ -114,7 +106,7 @@ fn struct_dict<'a>(
     vals: impl Iterator<Item = AnyValue<'a>>,
     flds: &[Field],
 ) -> PyObject {
-    let dict = PyDict::new(py);
+    let dict = PyDict::new_bound(py);
     for (fld, val) in flds.iter().zip(vals) {
         dict.set_item(fld.name().as_str(), Wrap(val)).unwrap()
     }
@@ -293,9 +285,9 @@ impl FromPyObject<'_> for Wrap<Field> {
 impl FromPyObject<'_> for Wrap<DataType> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
         let py = ob.py();
-        let type_name = ob.get_type().name()?;
+        let type_name = ob.get_type().qualname()?;
 
-        let dtype = match type_name {
+        let dtype = match &*type_name {
             "DataTypeClass" => {
                 // just the class, not an object
                 let name = ob.getattr(intern!(py, "__name__"))?.str()?.to_str()?;
@@ -352,7 +344,7 @@ impl FromPyObject<'_> for Wrap<DataType> {
             },
             "Enum" => {
                 let categories = ob.getattr(intern!(py, "categories")).unwrap();
-                let s = get_series(categories)?;
+                let s = get_series(&categories.as_borrowed())?;
                 let ca = s.str().map_err(PyPolarsErr::from)?;
                 let categories = ca.downcast_iter().next().unwrap().clone();
                 create_enum_data_type(categories)
@@ -435,8 +427,7 @@ impl ToPyObject for Wrap<TimeUnit> {
 impl<'s> FromPyObject<'s> for Wrap<Row<'s>> {
     fn extract(ob: &'s PyAny) -> PyResult<Self> {
         let vals = ob.extract::<Vec<Wrap<AnyValue<'s>>>>()?;
-        // SAFETY. Wrap is repr transparent.
-        let vals: Vec<AnyValue> = unsafe { std::mem::transmute(vals) };
+        let vals = vec_extract_wrapped(vals);
         Ok(Wrap(Row(vals)))
     }
 }
@@ -481,7 +472,7 @@ impl PartialEq for ObjectValue {
                 .as_ref(py)
                 .rich_compare(other.inner.as_ref(py), CompareOp::Eq)
             {
-                Ok(result) => result.is_true().unwrap(),
+                Ok(result) => result.is_truthy().unwrap(),
                 Err(_) => false,
             }
         })
@@ -556,7 +547,7 @@ impl Default for ObjectValue {
 
 impl<'a, T: NativeType + FromPyObject<'a>> FromPyObject<'a> for Wrap<Vec<T>> {
     fn extract(obj: &'a PyAny) -> PyResult<Self> {
-        let seq = <PySequence as PyTryFrom>::try_from(obj)?;
+        let seq = obj.downcast::<PySequence>()?;
         let mut v = Vec::with_capacity(seq.len().unwrap_or(0));
         for item in seq.iter()? {
             v.push(item?.extract::<T>()?);
@@ -565,50 +556,6 @@ impl<'a, T: NativeType + FromPyObject<'a>> FromPyObject<'a> for Wrap<Vec<T>> {
     }
 }
 
-pub(crate) fn dicts_to_rows(
-    records: &PyAny,
-    infer_schema_len: Option<usize>,
-    schema_columns: PlIndexSet<String>,
-) -> PyResult<(Vec<Row>, Vec<String>)> {
-    let infer_schema_len = infer_schema_len.map(|n| std::cmp::max(1, n));
-    let len = records.len()?;
-
-    let key_names = {
-        if !schema_columns.is_empty() {
-            schema_columns
-        } else {
-            let mut inferred_keys = PlIndexSet::new();
-            for d in records.iter()?.take(infer_schema_len.unwrap_or(usize::MAX)) {
-                let d = d?;
-                let d = d.downcast::<PyDict>()?;
-                let keys = d.keys();
-                for name in keys {
-                    let name = name.extract::<String>()?;
-                    inferred_keys.insert(name);
-                }
-            }
-            inferred_keys
-        }
-    };
-    let mut rows = Vec::with_capacity(len);
-
-    for d in records.iter()? {
-        let d = d?;
-        let d = d.downcast::<PyDict>()?;
-
-        let mut row = Vec::with_capacity(key_names.len());
-        for k in key_names.iter() {
-            let val = match d.get_item(k)? {
-                None => AnyValue::Null,
-                Some(val) => val.extract::<Wrap<AnyValue>>()?.0,
-            };
-            row.push(val)
-        }
-        rows.push(Row(row))
-    }
-    Ok((rows, key_names.into_iter().collect()))
-}
-
 #[cfg(feature = "asof_join")]
 impl FromPyObject<'_> for Wrap<AsofStrategy> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
@@ -912,6 +859,22 @@ impl FromPyObject<'_> for Wrap<RankMethod> {
     }
 }
 
+impl FromPyObject<'_> for Wrap<Roll> {
+    fn extract(ob: &PyAny) -> PyResult<Self> {
+        let parsed = match ob.extract::<&str>()? {
+            "raise" => Roll::Raise,
+            "forward" => Roll::Forward,
+            "backward" => Roll::Backward,
+            v => {
+                return Err(PyValueError::new_err(format!(
+                    "`roll` must be one of {{'raise', 'forward', 'backward'}}, got {v}",
+                )))
+            },
+        };
+        Ok(Wrap(parsed))
+    }
+}
+
 impl FromPyObject<'_> for Wrap<TimeUnit> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
         let parsed = match ob.extract::<&str>()? {
