@@ -32,7 +32,7 @@ impl std::convert::From<PyPolarsErr> for PyErr {
         let default = || PyRuntimeError::new_err(format!("{:?}", &err));
 
         use PyPolarsErr::*;
-        match &err {
+        match err {
             Polars(err) => match err {
                 PolarsError::ColumnNotFound(name) => ColumnNotFoundError::new_err(name.to_string()),
                 PolarsError::ComputeError(err) => ComputeError::new_err(err.to_string()),
@@ -40,11 +40,18 @@ impl std::convert::From<PyPolarsErr> for PyErr {
                 PolarsError::InvalidOperation(err) => {
                     InvalidOperationError::new_err(err.to_string())
                 },
-                PolarsError::Io(err) => match err.kind() {
-                    ErrorKind::NotFound => PyFileNotFoundError::new_err(err.to_string()),
-                    ErrorKind::PermissionDenied => PyPermissionError::new_err(err.to_string()),
-                    ErrorKind::AlreadyExists => PyFileExistsError::new_err(err.to_string()),
-                    _ => PyIOError::new_err(err.to_string()),
+                PolarsError::IO { error, msg } => {
+                    let msg = if let Some(msg) = msg {
+                        msg.to_string()
+                    } else {
+                        error.to_string()
+                    };
+                    match error.kind() {
+                        ErrorKind::NotFound => PyFileNotFoundError::new_err(msg),
+                        ErrorKind::PermissionDenied => PyPermissionError::new_err(msg),
+                        ErrorKind::AlreadyExists => PyFileExistsError::new_err(msg),
+                        _ => PyIOError::new_err(msg),
+                    }
                 },
                 PolarsError::NoData(err) => NoDataError::new_err(err.to_string()),
                 PolarsError::OutOfBounds(err) => OutOfBoundsError::new_err(err.to_string()),
@@ -59,6 +66,10 @@ impl std::convert::From<PyPolarsErr> for PyErr {
                 PolarsError::StructFieldNotFound(name) => {
                     StructFieldNotFoundError::new_err(name.to_string())
                 },
+                PolarsError::Context { .. } => {
+                    let tmp = PyPolarsErr::Polars(err.context_trace());
+                    PyErr::from(tmp)
+                },
             },
             _ => default(),
         }
@@ -94,6 +105,11 @@ create_exception!(
     CategoricalRemappingWarning,
     PolarsBaseWarning
 );
+create_exception!(
+    polars.exceptions,
+    MapWithoutReturnDtypeWarning,
+    PolarsBaseWarning
+);
 
 #[macro_export]
 macro_rules! raise_err(
@@ -109,6 +125,9 @@ impl IntoPy<PyObject> for Wrap<PolarsWarning> {
             PolarsWarning::CategoricalRemappingWarning => {
                 CategoricalRemappingWarning::type_object(py).to_object(py)
             },
+            PolarsWarning::MapWithoutReturnDtypeWarning => {
+                MapWithoutReturnDtypeWarning::type_object(py).to_object(py)
+            },
             PolarsWarning::UserWarning => PyUserWarning::type_object(py).to_object(py),
         }
     }
