@@ -44,8 +44,12 @@ impl PyExpr {
         self.inner.clone().list().eval(expr.inner, parallel).into()
     }
 
-    fn list_get(&self, index: PyExpr) -> Self {
-        self.inner.clone().list().get(index.inner).into()
+    fn list_get(&self, index: PyExpr, null_on_oob: bool) -> Self {
+        self.inner
+            .clone()
+            .list()
+            .get(index.inner, null_on_oob)
+            .into()
     }
 
     fn list_join(&self, separator: PyExpr, ignore_nulls: bool) -> Self {
@@ -128,11 +132,11 @@ impl PyExpr {
         self.inner
             .clone()
             .list()
-            .sort(SortOptions {
-                descending,
-                nulls_last,
-                ..Default::default()
-            })
+            .sort(
+                SortOptions::default()
+                    .with_order_descending(descending)
+                    .with_nulls_last(nulls_last),
+            )
             .into()
     }
 
