@@ -57,11 +57,25 @@ pub fn rolling_cov(
 }
 
 #[pyfunction]
-pub fn arg_sort_by(by: Vec<PyExpr>, descending: Vec<bool>) -> PyExpr {
+pub fn arg_sort_by(
+    by: Vec<PyExpr>,
+    descending: Vec<bool>,
+    nulls_last: bool,
+    multithreaded: bool,
+    maintain_order: bool,
+) -> PyExpr {
     let by = by.into_iter().map(|e| e.inner).collect::<Vec<Expr>>();
-    dsl::arg_sort_by(by, &descending).into()
+    dsl::arg_sort_by(
+        by,
+        SortMultipleOptions {
+            descending,
+            nulls_last,
+            multithreaded,
+            maintain_order,
+        },
+    )
+    .into()
 }
-
 #[pyfunction]
 pub fn arg_where(condition: PyExpr) -> PyExpr {
     dsl::arg_where(condition.inner).into()
@@ -104,35 +118,31 @@ pub fn collect_all(lfs: Vec<PyLazyFrame>, py: Python) -> PyResult<Vec<PyDataFram
 }
 
 #[pyfunction]
-pub fn collect_all_with_callback(lfs: Vec<PyLazyFrame>, lambda: PyObject, py: Python) {
+pub fn collect_all_with_callback(lfs: Vec<PyLazyFrame>, lambda: PyObject) {
     use polars_core::utils::rayon::prelude::*;
 
-    py.allow_threads(|| {
-        polars_core::POOL.install(move || {
-            polars_core::POOL.spawn(move || {
-                let result = lfs
-                    .par_iter()
-                    .map(|lf| {
-                        let df = lf.ldf.clone().collect()?;
-                        Ok(PyDataFrame::new(df))
-                    })
-                    .collect::<polars_core::error::PolarsResult<Vec<_>>>()
-                    .map_err(PyPolarsErr::from);
-
-                Python::with_gil(|py| match result {
-                    Ok(dfs) => {
-                        lambda.call1(py, (dfs,)).map_err(|err| err.restore(py)).ok();
-                    },
-                    Err(err) => {
-                        lambda
-                            .call1(py, (PyErr::from(err).to_object(py),))
-                            .map_err(|err| err.restore(py))
-                            .ok();
-                    },
-                })
+    polars_core::POOL.spawn(move || {
+        let result = lfs
+            .par_iter()
+            .map(|lf| {
+                let df = lf.ldf.clone().collect()?;
+                Ok(PyDataFrame::new(df))
             })
-        });
-    });
+            .collect::<polars_core::error::PolarsResult<Vec<_>>>()
+            .map_err(PyPolarsErr::from);
+
+        Python::with_gil(|py| match result {
+            Ok(dfs) => {
+                lambda.call1(py, (dfs,)).map_err(|err| err.restore(py)).ok();
+            },
+            Err(err) => {
+                lambda
+                    .call1(py, (PyErr::from(err).to_object(py),))
+                    .map_err(|err| err.restore(py))
+                    .ok();
+            },
+        })
+    })
 }
 
 #[pyfunction]
@@ -409,8 +419,10 @@ pub fn lit(value: &PyAny, allow_object: bool) -> PyResult<PyExpr> {
         Ok(dsl::lit(s).into())
     } else {
         Err(PyTypeError::new_err(format!(
-            "invalid literal value: {:?}",
-            value.str()?
+            "cannot create expression literal for value of type {}: {}\
+            \n\nHint: Pass `allow_object=True` to accept any value and create a literal of type Object.",
+            value.get_type().qualname()?,
+            value.repr()?
         )))
     }
 }
