@@ -6,10 +6,10 @@ use std::num::NonZeroUsize;
 use std::path::PathBuf;
 
 pub use exitable::PyInProcessQuery;
-use polars::io::RowIndex;
+use polars::io::cloud::CloudOptions;
+use polars::io::{HiveOptions, RowIndex};
 use polars::time::*;
 use polars_core::prelude::*;
-use polars_rs::io::cloud::CloudOptions;
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::{PyBytes, PyDict, PyList};
@@ -57,7 +57,7 @@ impl PyLazyFrame {
         // Used in pickle/pickling
         match state.extract::<&PyBytes>(py) {
             Ok(s) => {
-                let lp: LogicalPlan = ciborium::de::from_reader(s.as_bytes())
+                let lp: DslPlan = ciborium::de::from_reader(s.as_bytes())
                     .map_err(|e| PyPolarsErr::Other(format!("{}", e)))?;
                 self.ldf = LazyFrame::from(lp);
                 Ok(())
@@ -85,14 +85,14 @@ impl PyLazyFrame {
             .unwrap();
 
         // SAFETY:
-        // we skipped the serializing/deserializing of the static in lifetime in `DataType`
+        // We skipped the serializing/deserializing of the static in lifetime in `DataType`
         // so we actually don't have a lifetime at all when serializing.
 
         // &str still has a lifetime. But it's ok, because we drop it immediately
-        // in this scope
+        // in this scope.
         let json = unsafe { std::mem::transmute::<&'_ str, &'static str>(json.as_str()) };
 
-        let lp = serde_json::from_str::<LogicalPlan>(json)
+        let lp = serde_json::from_str::<DslPlan>(json)
             .map_err(|err| PyValueError::new_err(format!("{err:?}")))?;
         Ok(LazyFrame::from(lp).into())
     }
@@ -141,7 +141,7 @@ impl PyLazyFrame {
     #[pyo3(signature = (path, paths, separator, has_header, ignore_errors, skip_rows, n_rows, cache, overwrite_dtype,
         low_memory, comment_prefix, quote_char, null_values, missing_utf8_is_empty_string,
         infer_schema_length, with_schema_modify, rechunk, skip_rows_after_header,
-        encoding, row_index, try_parse_dates, eol_char, raise_if_empty, truncate_ragged_lines, schema
+        encoding, row_index, try_parse_dates, eol_char, raise_if_empty, truncate_ragged_lines, decimal_comma, schema
     )
     )]
     fn new_from_csv(
@@ -169,6 +169,7 @@ impl PyLazyFrame {
         eol_char: &str,
         raise_if_empty: bool,
         truncate_ragged_lines: bool,
+        decimal_comma: bool,
         schema: Option<Wrap<Schema>>,
     ) -> PyResult<Self> {
         let null_values = null_values.map(|w| w.0);
@@ -198,7 +199,7 @@ impl PyLazyFrame {
             .with_skip_rows(skip_rows)
             .with_n_rows(n_rows)
             .with_cache(cache)
-            .with_dtype_overwrite(overwrite_dtype.as_ref())
+            .with_dtype_overwrite(overwrite_dtype.map(Arc::new))
             .with_schema(schema.map(|schema| Arc::new(schema.0)))
             .low_memory(low_memory)
             .with_comment_prefix(comment_prefix)
@@ -212,6 +213,7 @@ impl PyLazyFrame {
             .with_null_values(null_values)
             .with_missing_is_null(!missing_utf8_is_empty_string)
             .truncate_ragged_lines(truncate_ragged_lines)
+            .with_decimal_comma(decimal_comma)
             .raise_if_empty(raise_if_empty);
 
         if let Some(lambda) = with_schema_modify {
@@ -243,7 +245,7 @@ impl PyLazyFrame {
     #[cfg(feature = "parquet")]
     #[staticmethod]
     #[pyo3(signature = (path, paths, n_rows, cache, parallel, rechunk, row_index,
-        low_memory, cloud_options, use_statistics, hive_partitioning, retries)
+        low_memory, cloud_options, use_statistics, hive_partitioning, hive_schema, retries)
     )]
     fn new_from_parquet(
         path: Option<PathBuf>,
@@ -257,8 +259,12 @@ impl PyLazyFrame {
         cloud_options: Option<Vec<(String, String)>>,
         use_statistics: bool,
         hive_partitioning: bool,
+        hive_schema: Option<Wrap<Schema>>,
         retries: usize,
     ) -> PyResult<Self> {
+        let parallel = parallel.0;
+        let hive_schema = hive_schema.map(|s| Arc::new(s.0));
+
         let first_path = if let Some(path) = &path {
             path
         } else {
@@ -281,16 +287,21 @@ impl PyLazyFrame {
                     });
         }
         let row_index = row_index.map(|(name, offset)| RowIndex { name, offset });
+        let hive_options = HiveOptions {
+            enabled: hive_partitioning,
+            schema: hive_schema,
+        };
+
         let args = ScanArgsParquet {
             n_rows,
             cache,
-            parallel: parallel.0,
+            parallel,
             rechunk,
             row_index,
             low_memory,
             cloud_options,
             use_statistics,
-            hive_partitioning,
+            hive_options,
         };
 
         let lf = if path.is_some() {
@@ -349,7 +360,7 @@ impl PyLazyFrame {
             cache,
             rechunk,
             row_index,
-            memmap: memory_map,
+            memory_map,
             #[cfg(feature = "cloud")]
             cloud_options,
         };
@@ -449,14 +460,15 @@ impl PyLazyFrame {
         descending: bool,
         nulls_last: bool,
         maintain_order: bool,
+        multithreaded: bool,
     ) -> Self {
         let ldf = self.ldf.clone();
         ldf.sort(
-            by_column,
-            SortOptions {
-                descending,
+            [by_column],
+            SortMultipleOptions {
+                descending: vec![descending],
                 nulls_last,
-                multithreaded: true,
+                multithreaded,
                 maintain_order,
             },
         )
@@ -469,11 +481,20 @@ impl PyLazyFrame {
         descending: Vec<bool>,
         nulls_last: bool,
         maintain_order: bool,
+        multithreaded: bool,
     ) -> Self {
         let ldf = self.ldf.clone();
         let exprs = by.to_exprs();
-        ldf.sort_by_exprs(exprs, descending, nulls_last, maintain_order)
-            .into()
+        ldf.sort_by_exprs(
+            exprs,
+            SortMultipleOptions {
+                descending,
+                nulls_last,
+                maintain_order,
+                multithreaded,
+            },
+        )
+        .into()
     }
 
     fn top_k(
@@ -483,11 +504,21 @@ impl PyLazyFrame {
         descending: Vec<bool>,
         nulls_last: bool,
         maintain_order: bool,
+        multithreaded: bool,
     ) -> Self {
         let ldf = self.ldf.clone();
         let exprs = by.to_exprs();
-        ldf.top_k(k, exprs, descending, nulls_last, maintain_order)
-            .into()
+        ldf.top_k(
+            k,
+            exprs,
+            SortMultipleOptions {
+                descending,
+                nulls_last,
+                maintain_order,
+                multithreaded,
+            },
+        )
+        .into()
     }
 
     fn bottom_k(
@@ -497,11 +528,21 @@ impl PyLazyFrame {
         descending: Vec<bool>,
         nulls_last: bool,
         maintain_order: bool,
+        multithreaded: bool,
     ) -> Self {
         let ldf = self.ldf.clone();
         let exprs = by.to_exprs();
-        ldf.bottom_k(k, exprs, descending, nulls_last, maintain_order)
-            .into()
+        ldf.bottom_k(
+            k,
+            exprs,
+            SortMultipleOptions {
+                descending,
+                nulls_last,
+                maintain_order,
+                multithreaded,
+            },
+        )
+        .into()
     }
 
     fn cache(&self) -> Self {
@@ -530,27 +571,25 @@ impl PyLazyFrame {
     }
 
     #[pyo3(signature = (lambda,))]
-    fn collect_with_callback(&self, py: Python, lambda: PyObject) {
-        py.allow_threads(|| {
-            let ldf = self.ldf.clone();
+    fn collect_with_callback(&self, lambda: PyObject) {
+        let ldf = self.ldf.clone();
 
-            polars_core::POOL.spawn(move || {
-                let result = ldf
-                    .collect()
-                    .map(PyDataFrame::new)
-                    .map_err(PyPolarsErr::from);
+        polars_core::POOL.spawn(move || {
+            let result = ldf
+                .collect()
+                .map(PyDataFrame::new)
+                .map_err(PyPolarsErr::from);
 
-                Python::with_gil(|py| match result {
-                    Ok(df) => {
-                        lambda.call1(py, (df,)).map_err(|err| err.restore(py)).ok();
-                    },
-                    Err(err) => {
-                        lambda
-                            .call1(py, (PyErr::from(err).to_object(py),))
-                            .map_err(|err| err.restore(py))
-                            .ok();
-                    },
-                });
+            Python::with_gil(|py| match result {
+                Ok(df) => {
+                    lambda.call1(py, (df,)).map_err(|err| err.restore(py)).ok();
+                },
+                Err(err) => {
+                    lambda
+                        .call1(py, (PyErr::from(err).to_object(py),))
+                        .map_err(|err| err.restore(py))
+                        .ok();
+                },
             });
         });
     }
@@ -727,7 +766,7 @@ impl PyLazyFrame {
             .into_iter()
             .map(|pyexpr| pyexpr.inner)
             .collect::<Vec<_>>();
-        let lazy_gb = ldf.group_by_rolling(
+        let lazy_gb = ldf.rolling(
             index_column.inner,
             by,
             RollingGroupOptions {
@@ -751,19 +790,19 @@ impl PyLazyFrame {
         label: Wrap<Label>,
         include_boundaries: bool,
         closed: Wrap<ClosedWindow>,
-        by: Vec<PyExpr>,
+        group_by: Vec<PyExpr>,
         start_by: Wrap<StartBy>,
         check_sorted: bool,
     ) -> PyLazyGroupBy {
         let closed_window = closed.0;
-        let by = by
+        let group_by = group_by
             .into_iter()
             .map(|pyexpr| pyexpr.inner)
             .collect::<Vec<_>>();
         let ldf = self.ldf.clone();
         let lazy_gb = ldf.group_by_dynamic(
             index_column.inner,
-            by,
+            group_by,
             DynamicGroupOptions {
                 every: Duration::parse(every),
                 period: Duration::parse(period),
@@ -901,58 +940,52 @@ impl PyLazyFrame {
         ldf.fill_nan(fill_value.inner).into()
     }
 
-    fn min(&self) -> PyResult<Self> {
+    fn min(&self) -> Self {
         let ldf = self.ldf.clone();
-        let out = ldf.min().map_err(PyPolarsErr::from)?;
-        Ok(out.into())
+        let out = ldf.min();
+        out.into()
     }
 
-    fn max(&self) -> PyResult<Self> {
+    fn max(&self) -> Self {
         let ldf = self.ldf.clone();
-        let out = ldf.max().map_err(PyPolarsErr::from)?;
-        Ok(out.into())
+        let out = ldf.max();
+        out.into()
     }
 
-    fn sum(&self) -> PyResult<Self> {
+    fn sum(&self) -> Self {
         let ldf = self.ldf.clone();
-        let out = ldf.sum().map_err(PyPolarsErr::from)?;
-        Ok(out.into())
+        let out = ldf.sum();
+        out.into()
     }
 
-    fn mean(&self) -> PyResult<Self> {
+    fn mean(&self) -> Self {
         let ldf = self.ldf.clone();
-        let out = ldf.mean().map_err(PyPolarsErr::from)?;
-        Ok(out.into())
+        let out = ldf.mean();
+        out.into()
     }
 
-    fn std(&self, ddof: u8) -> PyResult<Self> {
+    fn std(&self, ddof: u8) -> Self {
         let ldf = self.ldf.clone();
-        let out = ldf.std(ddof).map_err(PyPolarsErr::from)?;
-        Ok(out.into())
+        let out = ldf.std(ddof);
+        out.into()
     }
 
-    fn var(&self, ddof: u8) -> PyResult<Self> {
+    fn var(&self, ddof: u8) -> Self {
         let ldf = self.ldf.clone();
-        let out = ldf.var(ddof).map_err(PyPolarsErr::from)?;
-        Ok(out.into())
+        let out = ldf.var(ddof);
+        out.into()
     }
 
-    fn median(&self) -> PyResult<Self> {
+    fn median(&self) -> Self {
         let ldf = self.ldf.clone();
-        let out = ldf.median().map_err(PyPolarsErr::from)?;
-        Ok(out.into())
+        let out = ldf.median();
+        out.into()
     }
 
-    fn quantile(
-        &self,
-        quantile: PyExpr,
-        interpolation: Wrap<QuantileInterpolOptions>,
-    ) -> PyResult<Self> {
+    fn quantile(&self, quantile: PyExpr, interpolation: Wrap<QuantileInterpolOptions>) -> Self {
         let ldf = self.ldf.clone();
-        let out = ldf
-            .quantile(quantile.inner, interpolation.0)
-            .map_err(PyPolarsErr::from)?;
-        Ok(out.into())
+        let out = ldf.quantile(quantile.inner, interpolation.0);
+        out.into()
     }
 
     fn explode(&self, column: Vec<PyExpr>) -> Self {
