@@ -9,6 +9,7 @@ import pytest
 import polars as pl
 import polars.selectors as cs
 from polars.testing import assert_frame_equal, assert_series_equal
+from polars.testing._constants import PARTITION_LIMIT
 
 if TYPE_CHECKING:
     from polars.type_aliases import PolarsDataType
@@ -675,7 +676,7 @@ def test_group_by_multiple_column_reference() -> None:
         ("mean", [], [1.0, None], pl.Float64),
         ("median", [], [1.0, None], pl.Float64),
         ("min", [], [1, None], pl.Int64),
-        ("n_unique", [], [1, None], pl.UInt32),
+        ("n_unique", [], [1, 0], pl.UInt32),
         ("quantile", [0.5], [1.0, None], pl.Float64),
     ],
 )
@@ -768,6 +769,13 @@ def test_group_by_partitioned_ending_cast(monkeypatch: Any) -> None:
     assert_frame_equal(out, expected)
 
 
+def test_group_by_series_partitioned() -> None:
+    # test 15354
+    df = pl.DataFrame([0, 0] * PARTITION_LIMIT)
+    groups = pl.Series([0, 1] * PARTITION_LIMIT)
+    df.group_by(groups).agg(pl.all().is_not_null().sum())
+
+
 def test_groupby_deprecated() -> None:
     df = pl.DataFrame({"a": [1, 1, 2], "b": [3, 4, 5]})
 
@@ -967,3 +975,18 @@ def test_partitioned_group_by_14954(monkeypatch: Any) -> None:
             [False, False, False, False, False, False, False, False, False, False],
         ],
     }
+
+
+def test_aggregated_scalar_elementwise_15602() -> None:
+    df = pl.DataFrame({"group": [1, 2, 1]})
+
+    out = df.group_by("group", maintain_order=True).agg(
+        foo=pl.col("group").is_between(1, pl.max("group"))
+    )
+    expected = pl.DataFrame({"group": [1, 2], "foo": [[True, True], [True]]})
+    assert_frame_equal(out, expected)
+
+
+def test_group_by_multiple_null_cols_15623() -> None:
+    df = pl.DataFrame(schema={"a": pl.Null, "b": pl.Null}).group_by(pl.all()).len()
+    assert df.is_empty()
