@@ -384,18 +384,30 @@ def test_jit_sort_joins() -> None:
         pd_result.columns = pd.Index(["a", "b", "b_right"])
 
         # left key sorted right is not
-        pl_result = dfa_pl.join(dfb_pl, on="a", how=how).sort(["a", "b"])
+        pl_result = dfa_pl.join(dfb_pl, on="a", how=how).sort(
+            ["a", "b"], maintain_order=True
+        )
 
-        a = pl.from_pandas(pd_result).with_columns(pl.all().cast(int)).sort(["a", "b"])
+        a = (
+            pl.from_pandas(pd_result)
+            .with_columns(pl.all().cast(int))
+            .sort(["a", "b"], maintain_order=True)
+        )
         assert_frame_equal(a, pl_result)
         assert pl_result["a"].flags["SORTED_ASC"]
 
         # left key sorted right is not
         pd_result = dfb.merge(dfa, on="a", how=how)
         pd_result.columns = pd.Index(["a", "b", "b_right"])
-        pl_result = dfb_pl.join(dfa_pl, on="a", how=how).sort(["a", "b"])
+        pl_result = dfb_pl.join(dfa_pl, on="a", how=how).sort(
+            ["a", "b"], maintain_order=True
+        )
 
-        a = pl.from_pandas(pd_result).with_columns(pl.all().cast(int)).sort(["a", "b"])
+        a = (
+            pl.from_pandas(pd_result)
+            .with_columns(pl.all().cast(int))
+            .sort(["a", "b"], maintain_order=True)
+        )
         assert_frame_equal(a, pl_result)
         assert pl_result["a"].flags["SORTED_ASC"]
 
@@ -803,3 +815,57 @@ def test_join_results_in_duplicate_names() -> None:
     rhs = lhs.clone()
     with pytest.raises(pl.DuplicateError, match="'c_right' already exists"):
         lhs.join(rhs, on=["a", "b"], how="left")
+
+
+def test_join_projection_invalid_name_contains_suffix_15243() -> None:
+    df1 = pl.DataFrame({"a": [1, 2, 3]}).lazy()
+    df2 = pl.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]}).lazy()
+
+    with pytest.raises(pl.ColumnNotFoundError):
+        (
+            df1.join(df2, on="a")
+            .select(pl.col("b").filter(pl.col("b") == pl.col("foo_right")))
+            .collect()
+        )
+
+
+def test_join_list_non_numeric() -> None:
+    assert (
+        pl.DataFrame(
+            {
+                "lists": [
+                    ["a", "b", "c"],
+                    ["a", "c", "b"],
+                    ["a", "c", "b"],
+                    ["a", "c", "d"],
+                ]
+            }
+        )
+    ).group_by("lists", maintain_order=True).agg(pl.len().alias("count")).to_dict(
+        as_series=False
+    ) == {
+        "lists": [["a", "b", "c"], ["a", "c", "b"], ["a", "c", "d"]],
+        "count": [1, 2, 1],
+    }
+
+
+@pytest.mark.slow()
+def test_join_4_columns_with_validity() -> None:
+    # join on 4 columns so we trigger combine validities
+    # use 138 as that is 2 u64 and a remainder
+    a = pl.DataFrame(
+        {"a": [None if a % 6 == 0 else a for a in range(138)]}
+    ).with_columns(
+        b=pl.col("a"),
+        c=pl.col("a"),
+        d=pl.col("a"),
+    )
+
+    assert a.join(a, on=["a", "b", "c", "d"], how="inner", join_nulls=True).shape == (
+        644,
+        4,
+    )
+    assert a.join(a, on=["a", "b", "c", "d"], how="inner", join_nulls=False).shape == (
+        115,
+        4,
+    )
