@@ -193,7 +193,7 @@ def test_asof_join_projection_() -> None:
         pl.DataFrame(
             {
                 "group": [0, 2, 3, 0, 1, 2, 3],
-                "val": [0, 2.5, 2.6, 2.7, 3.4, 4, 5],
+                "val": [0.0, 2.5, 2.6, 2.7, 3.4, 4.0, 5.0],
                 "c": ["x", "x", "x", "y", "y", "y", "y"],
             }
         )
@@ -408,3 +408,14 @@ def test_projection_drop_with_series_lit_14382() -> None:
         "b_name": ["b"],
         "b_old": [7],
     }
+
+
+def test_cached_schema_15651() -> None:
+    q = pl.LazyFrame({"col1": [1], "col2": [2], "col3": [3]})
+    q = q.with_row_index()
+    q = q.filter(~pl.col("col1").is_null())
+    # create a subplan diverging from q
+    _ = q.select(pl.len()).collect(projection_pushdown=True)
+
+    # ensure that q's "cached" columns are still correct
+    assert q.columns == q.collect().columns
